@@ -1,8 +1,8 @@
 /*
  *      pluginsignals.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: pluginsignals.c 5710 2011-04-11 14:40:27Z ntrel $
  */
 
 /* Note: this file is for Doxygen only. */
@@ -158,18 +156,18 @@ signal void (*project_save)(GObject *obj, GKeyFile *config, gpointer user_data);
  */
 signal void (*project_close)(GObject *obj, gpointer user_data);
 
-/** Sent after a project dialog is created but before it is displayed. Plugins
+/** Sent after a project dialog is opened but before it is displayed. Plugins
  *  can append their own project settings tabs by using this signal.
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param notebook a GtkNotebook instance that can be used by plugins to append their
  *  settings tabs.
  *  @param user_data user data.
  */
-signal void (*project_dialog_create)(GObject *obj, GtkWidget *notebook, gpointer user_data);
+signal void (*project_dialog_open)(GObject *obj, GtkWidget *notebook, gpointer user_data);
 
 /** Sent when the settings dialog is confirmed by the user. Plugins can use
  *  this signal to read the settings widgets previously added by using the
- *  @c project-dialog-create signal.
+ *  @c project-dialog-open signal.
  *  @warning The dialog will still be running afterwards if the user chose 'Apply'.
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param notebook a GtkNotebook instance that can be used by plugins to read their
@@ -178,6 +176,15 @@ signal void (*project_dialog_create)(GObject *obj, GtkWidget *notebook, gpointer
  */
 signal void (*project_dialog_confirmed)(GObject *obj, GtkWidget *notebook, gpointer user_data);
 
+/** Sent before project dialog is closed. By using this signal, plugins can remove 
+ *  tabs previously added in project-dialog-open signal handler.
+ *  @param obj a GeanyObject instance, should be ignored.
+ *  @param notebook a GtkNotebook instance that can be used by plugins to remove
+ *  settings tabs previously added in the project-dialog-open signal handler.
+ *  @param user_data user data.
+ */
+signal void (*project_dialog_close)(GObject *obj, GtkWidget *notebook, gpointer user_data);
+
 /** Sent once Geany has finished all initialization and startup tasks and the GUI has been
  *  realized. This signal is the very last step in the startup process and is sent once
  *  the GTK main event loop has been entered.
@@ -187,7 +194,7 @@ signal void (*project_dialog_confirmed)(GObject *obj, GtkWidget *notebook, gpoin
  */
 signal void (*geany_startup_complete)(GObject *obj, gpointer user_data);
 
-/** Sent before build is started. Plugins can use this signal e.g. to save the opened documents
+/** Sent before build is started. A plugin could use this signal e.g. to save all unsaved documents
  *  before the build starts.
  *
  *  @param obj a GeanyObject instance, should be ignored.
