@@ -1,8 +1,8 @@
 /*
  *      filebrowser.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: filebrowser.c 5882 2011-08-03 15:20:26Z colombanw $
  */
 
 /* Sidebar file browser plugin. */
@@ -243,7 +241,7 @@ static void add_top_level_entry(void)
 		return;	/* ignore 'C:\' or '/' */
 
 	utf8_dir = g_path_get_dirname(current_dir);
-	setptr(utf8_dir, utils_get_utf8_from_locale(utf8_dir));
+	SETPTR(utf8_dir, utils_get_utf8_from_locale(utf8_dir));
 
 	gtk_list_store_prepend(file_store, &iter);
 	last_dir_iter = gtk_tree_iter_copy(&iter);
@@ -282,6 +280,7 @@ static void refresh(void)
 
 	utf8_dir = utils_get_utf8_from_locale(current_dir);
 	gtk_entry_set_text(GTK_ENTRY(path_entry), utf8_dir);
+	gtk_widget_set_tooltip_text(path_entry, utf8_dir);
 	ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(path_combo), utf8_dir, 0);
 	g_free(utf8_dir);
 
@@ -306,7 +305,7 @@ static void refresh(void)
 
 static void on_go_home(void)
 {
-	setptr(current_dir, g_strdup(g_get_home_dir()));
+	SETPTR(current_dir, g_strdup(g_get_home_dir()));
 	refresh();
 }
 
@@ -337,7 +336,7 @@ static void on_current_path(void)
 
 	if (doc == NULL || doc->file_name == NULL || ! g_path_is_absolute(doc->file_name))
 	{
-		setptr(current_dir, get_default_dir());
+		SETPTR(current_dir, get_default_dir());
 		refresh();
 		return;
 	}
@@ -346,7 +345,7 @@ static void on_current_path(void)
 	dir = g_path_get_dirname(fname);
 	g_free(fname);
 
-	setptr(current_dir, dir);
+	SETPTR(current_dir, dir);
 	refresh();
 }
 
@@ -357,7 +356,7 @@ static void on_go_up(void)
 	if (current_dir[len-1] == G_DIR_SEPARATOR)
 		current_dir[len-1] = '\0';
 	/* remove the highest directory part (which becomes the basename of current_dir) */
-	setptr(current_dir, g_path_get_dirname(current_dir));
+	SETPTR(current_dir, g_path_get_dirname(current_dir));
 	refresh();
 }
 
@@ -513,7 +512,7 @@ static void open_folder(GtkTreePath *treepath)
 {
 	gchar *fname = get_tree_path_filename(treepath);
 
-	setptr(current_dir, fname);
+	SETPTR(current_dir, fname);
 	refresh();
 }
 
@@ -576,7 +575,7 @@ static void on_find_in_files(GtkMenuItem *menuitem, gpointer user_data)
 	g_list_foreach(list, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free(list);
 
-	setptr(dir, utils_get_utf8_from_locale(dir));
+	SETPTR(dir, utils_get_utf8_from_locale(dir));
 	search_show_find_in_files_dialog(dir);
 	g_free(dir);
 }
@@ -783,7 +782,7 @@ static void on_path_entry_activate(GtkEntry *entry, gpointer user_data)
 	else
 		new_dir = g_strdup(g_get_home_dir());
 
-	setptr(current_dir, new_dir);
+	SETPTR(current_dir, new_dir);
 
 	on_clear_filter(NULL, NULL);
 }
@@ -793,7 +792,7 @@ static void ui_combo_box_changed(GtkComboBox *combo, gpointer user_data)
 {
 	/* we get this callback on typing as well as choosing an item */
 	if (gtk_combo_box_get_active(combo) >= 0)
-		gtk_widget_activate(GTK_BIN(combo)->child);
+		gtk_widget_activate(gtk_bin_get_child(GTK_BIN(combo)));
 }
 
 
@@ -888,16 +887,6 @@ static GtkWidget *make_toolbar(void)
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_current_path), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
-	if (gtk_check_version(2, 15, 2) != NULL)
-	{
-		wid = GTK_WIDGET(gtk_separator_tool_item_new());
-		gtk_container_add(GTK_CONTAINER(toolbar), wid);
-
-		wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR));
-		gtk_widget_set_tooltip_text(wid, _("Clear the filter"));
-		g_signal_connect(wid, "clicked", G_CALLBACK(on_clear_filter), NULL);
-		gtk_container_add(GTK_CONTAINER(toolbar), wid);
-	}
 	return toolbar;
 }
 
@@ -911,13 +900,11 @@ static GtkWidget *make_filterbar(void)
 	label = gtk_label_new(_("Filter:"));
 
 	filter_combo = gtk_combo_box_entry_new_text();
-	filter_entry = GTK_BIN(filter_combo)->child;
+	filter_entry = gtk_bin_get_child(GTK_BIN(filter_combo));
+
+	ui_entry_add_clear_icon(GTK_ENTRY(filter_entry));
+	g_signal_connect(filter_entry, "icon-release", G_CALLBACK(on_filter_clear), NULL);
 
-	if (gtk_check_version(2, 15, 2) == NULL)
-	{
-		ui_entry_add_clear_icon(GTK_ENTRY(filter_entry));
-		g_signal_connect(filter_entry, "icon-release", G_CALLBACK(on_filter_clear), NULL);
-	}
 	gtk_widget_set_tooltip_text(filter_entry,
 		_("Filter your files with the usual wildcards. Separate multiple patterns with a space."));
 	g_signal_connect(filter_entry, "activate", G_CALLBACK(on_filter_activate), NULL);
@@ -1034,11 +1021,11 @@ static void project_change_cb(G_GNUC_UNUSED GObject *obj, G_GNUC_UNUSED GKeyFile
 		g_free(dir);
 	}
 	/* get it into locale encoding */
-	setptr(new_dir, utils_get_locale_from_utf8(new_dir));
+	SETPTR(new_dir, utils_get_locale_from_utf8(new_dir));
 
 	if (! utils_str_equal(current_dir, new_dir))
 	{
-		setptr(current_dir, new_dir);
+		SETPTR(current_dir, new_dir);
 		refresh();
 	}
 	else
@@ -1059,11 +1046,11 @@ static void document_activate_cb(G_GNUC_UNUSED GObject *obj, GeanyDocument *doc,
 		return;
 
 	new_dir = g_path_get_dirname(doc->file_name);
-	setptr(new_dir, utils_get_locale_from_utf8(new_dir));
+	SETPTR(new_dir, utils_get_locale_from_utf8(new_dir));
 
 	if (! utils_str_equal(current_dir, new_dir))
 	{
-		setptr(current_dir, new_dir);
+		SETPTR(current_dir, new_dir);
 		refresh();
 	}
 	else
@@ -1109,7 +1096,7 @@ void plugin_init(GeanyData *data)
 	path_combo = gtk_combo_box_entry_new_text();
 	gtk_box_pack_start(GTK_BOX(file_view_vbox), path_combo, FALSE, FALSE, 2);
 	g_signal_connect(path_combo, "changed", G_CALLBACK(ui_combo_box_changed), NULL);
-	path_entry = GTK_BIN(path_combo)->child;
+	path_entry = gtk_bin_get_child(GTK_BIN(path_combo));
 	g_signal_connect(path_entry, "activate", G_CALLBACK(on_path_entry_activate), NULL);
 
 	file_view = gtk_tree_view_new();
