@@ -1,8 +1,8 @@
 /*
  *      saveactions.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: saveactions.c 5882 2011-08-03 15:20:26Z colombanw $
  */
 
 
@@ -109,7 +107,7 @@ static gboolean backupcopy_set_backup_dir(const gchar *utf8_dir)
 	}
 	/** TODO add utils_is_file_writeable() to the plugin API and make use of it **/
 
-	setptr(backupcopy_backup_dir, tmp);
+	SETPTR(backupcopy_backup_dir, tmp);
 
 	return TRUE;
 }
@@ -261,7 +259,7 @@ static void instantsave_document_new_cb(GObject *obj, GeanyDocument *doc, gpoint
 
 		if (ft != NULL)
 			/* add the filetype's default extension to the new filename */
-			setptr(new_filename, g_strconcat(new_filename, ".", ft->extension, NULL));
+			SETPTR(new_filename, g_strconcat(new_filename, ".", ft->extension, NULL));
 
 		doc->file_name = new_filename;
 
@@ -298,8 +296,8 @@ static gboolean auto_save(gpointer data)
 		{
 			doc = document_get_from_page(i);
 
-			/* skip current file to save it lastly, skip files without name */
-			if (doc != cur_doc && cur_doc->file_name != NULL)
+			/* skip current file (save it last), skip files without name */
+			if (doc != cur_doc && doc->file_name != NULL)
 				if (document_save_file(doc, FALSE))
 					saved_files++;
 		}
@@ -456,7 +454,7 @@ static void configure_response_cb(GtkDialog *dialog, gint response, G_GNUC_UNUSE
 
 		g_key_file_set_integer(config, "backupcopy", "dir_levels", backupcopy_dir_levels);
 		g_key_file_set_string(config, "backupcopy", "time_fmt", text_time);
-		setptr(backupcopy_time_fmt, g_strdup(text_time));
+		SETPTR(backupcopy_time_fmt, g_strdup(text_time));
 		if (enable_backupcopy)
 		{
 			if (NZV(text_dir) && backupcopy_set_backup_dir(text_dir))
@@ -673,7 +671,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 		g_signal_connect(button, "clicked",
 			G_CALLBACK(backupcopy_dir_button_clicked_cb), entry_dir);
 
-		image = gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_BUTTON);
+		image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
 		gtk_container_add(GTK_CONTAINER(button), image);
 
 		hbox = gtk_hbox_new(FALSE, 6);
