@@ -1,8 +1,8 @@
 /*
  *      dialogs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: dialogs.c 5945 2011-09-22 12:05:05Z ntrel $
  */
 
 /*
@@ -148,7 +146,7 @@ static void open_file_dialog_handle_response(GtkWidget *dialog, gint response)
 
 		/* ignore detect from file item */
 		if (filesel_state.open.filetype_idx > 0)
-			ft = g_slist_nth_data(filetypes_by_title, filesel_state.open.filetype_idx);
+			ft = g_slist_nth_data(filetypes_by_title, (guint) filesel_state.open.filetype_idx);
 
 		encoding_model = gtk_combo_box_get_model(GTK_COMBO_BOX(encoding_combo));
 		gtk_combo_box_get_active_iter(GTK_COMBO_BOX(encoding_combo), &encoding_iter);
@@ -170,16 +168,15 @@ static void open_file_dialog_handle_response(GtkWidget *dialog, gint response)
 }
 
 
-static void on_file_open_notify(GObject *filechooser, GParamSpec *pspec, gpointer data)
+static void on_file_open_show_hidden_notify(GObject *filechooser,
+	GParamSpec *pspec, gpointer data)
 {
-	GValue *value;
+	GtkWidget *toggle_button;
 
-	value = g_new0(GValue, 1);
-	g_value_init(value, pspec->value_type);
-	g_object_get_property(filechooser, pspec->name, value);
+	toggle_button = ui_lookup_widget(GTK_WIDGET(filechooser), "check_hidden");
 
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
-		ui_lookup_widget(GTK_WIDGET(filechooser), "check_hidden")), g_value_get_boolean(value));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button),
+		gtk_file_chooser_get_show_hidden(GTK_FILE_CHOOSER(filechooser)));
 }
 
 
@@ -381,8 +378,7 @@ static GtkWidget *create_open_file_dialog(void)
 	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(main_widgets.window));
 	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
-	if (gtk_check_version(2, 14, 0) == NULL)
-		gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
+	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
 
 	/* add checkboxes and filename entry */
 	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), add_file_open_extra_widget(dialog));
@@ -417,7 +413,8 @@ static GtkWidget *create_open_file_dialog(void)
 	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(encoding_combo), encoding_renderer,
 			encoding_combo_cell_data_func, NULL, NULL);
 
-	g_signal_connect(dialog, "notify::show-hidden", G_CALLBACK(on_file_open_notify), NULL);
+	g_signal_connect(dialog, "notify::show-hidden",
+		G_CALLBACK(on_file_open_show_hidden_notify), NULL);
 
 	return dialog;
 }
@@ -462,7 +459,7 @@ void dialogs_show_open_file(void)
 	if (! initdir)
 		initdir = g_strdup(utils_get_default_dir_utf8());
 
-	setptr(initdir, utils_get_locale_from_utf8(initdir));
+	SETPTR(initdir, utils_get_locale_from_utf8(initdir));
 
 #ifdef G_OS_WIN32
 	if (interface_prefs.use_native_windows_dialogs)
@@ -608,8 +605,7 @@ static GtkWidget *create_save_file_dialog(void)
 	gtk_widget_show_all(vbox);
 	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), vbox);
 	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
-	if (gtk_check_version(2, 14, 0) == NULL)
-		gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
+	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
 
 	/* set the folder by default to the project base dir or the global pref for opening files */
 	initdir = utils_get_default_dir_utf8();
@@ -629,10 +625,9 @@ static GtkWidget *create_save_file_dialog(void)
 }
 
 
-static gboolean show_save_as_gtk(void)
+static gboolean show_save_as_gtk(GeanyDocument *doc)
 {
 	GtkWidget *dialog;
-	GeanyDocument *doc = document_get_current();
 	gint resp;
 
 	g_return_val_if_fail(doc != NULL, FALSE);
@@ -700,12 +695,14 @@ static gboolean show_save_as_gtk(void)
  **/
 gboolean dialogs_show_save_as()
 {
+	GeanyDocument *doc = document_get_current();
 	gboolean result = FALSE;
 
+	g_return_val_if_fail(doc, FALSE);
+
 #ifdef G_OS_WIN32
 	if (interface_prefs.use_native_windows_dialogs)
 	{
-		GeanyDocument *doc = document_get_current();
 		gchar *utf8_name = win32_show_document_save_as_dialog(GTK_WINDOW(main_widgets.window),
 						_("Save File"), DOC_FILENAME(doc));
 		if (utf8_name != NULL)
@@ -713,7 +710,7 @@ gboolean dialogs_show_save_as()
 	}
 	else
 #endif
-	result = show_save_as_gtk();
+	result = show_save_as_gtk(doc);
 	return result;
 }
 
@@ -804,7 +801,6 @@ void dialogs_show_msgbox_with_secondary(GtkMessageType type, const gchar *text,
 }
 
 
-#ifndef G_OS_WIN32
 static gint run_unsaved_dialog(const gchar *msg, const gchar *msg2)
 {
 	GtkWidget *dialog, *button;
@@ -812,6 +808,7 @@ static gint run_unsaved_dialog(const gchar *msg, const gchar *msg2)
 
 	dialog = gtk_message_dialog_new(GTK_WINDOW(main_widgets.window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", msg);
+	gtk_window_set_title(GTK_WINDOW(dialog), _("Question"));
 	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", msg2);
 	gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 
@@ -828,56 +825,42 @@ static gint run_unsaved_dialog(const gchar *msg, const gchar *msg2)
 
 	return ret;
 }
-#endif
 
 
 gboolean dialogs_show_unsaved_file(GeanyDocument *doc)
 {
 	gchar *msg, *short_fn = NULL;
 	const gchar *msg2;
-	gint ret;
+	gint response;
 	gboolean old_quitting_state = main_status.quitting;
 
 	/* display the file tab to remind the user of the document */
 	main_status.quitting = FALSE;
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-		document_get_notebook_page(doc));
+	document_show_tab(doc);
 	main_status.quitting = old_quitting_state;
 
 	short_fn = document_get_basename_for_display(doc, -1);
 
-	msg = g_strdup_printf(_("The file '%s' is not saved."),
-		(short_fn != NULL) ? short_fn : GEANY_STRING_UNTITLED);
+	msg = g_strdup_printf(_("The file '%s' is not saved."), short_fn);
 	msg2 = _("Do you want to save it before closing?");
 	g_free(short_fn);
 
-#ifdef G_OS_WIN32
-	setptr(msg, g_strconcat(msg, "\n", msg2, NULL));
-	ret = win32_message_dialog_unsaved(msg);
-#else
-	ret = run_unsaved_dialog(msg, msg2);
-#endif
+	response = run_unsaved_dialog(msg, msg2);
 	g_free(msg);
 
-	switch (ret)
+	switch (response)
 	{
 		case GTK_RESPONSE_YES:
-		{
-			if (document_need_save_as(doc))
-			{
-				ret = dialogs_show_save_as();
-			}
-			else
-				/* document_save_file() returns the status if the file could be saved */
-				ret = document_save_file(doc, FALSE);
-			break;
-		}
-		case GTK_RESPONSE_NO: ret = TRUE; break;
-		case GTK_RESPONSE_CANCEL: /* fall through to default and leave the function */
-		default: ret = FALSE; break;
-	}
+			/* document_save_file() returns the status if the file could be saved */
+			return document_save_file(doc, FALSE);
 
-	return (gboolean) ret;
+		case GTK_RESPONSE_NO:
+			return TRUE;
+
+		case GTK_RESPONSE_CANCEL: /* fall through to default and leave the function */
+		default:
+			return FALSE;
+	}
 }
 
 
@@ -939,10 +922,10 @@ void dialogs_show_open_font()
 		g_signal_connect(GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel)->apply_button,
 					"clicked", G_CALLBACK(on_font_apply_button_clicked), NULL);
 
-		gtk_font_selection_dialog_set_font_name(
-			GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel), interface_prefs.editor_font);
 		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.open_fontsel), GTK_WINDOW(main_widgets.window));
 	}
+	gtk_font_selection_dialog_set_font_name(
+		GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel), interface_prefs.editor_font);
 	/* We make sure the dialog is visible. */
 	gtk_window_present(GTK_WINDOW(ui_widgets.open_fontsel));
 #endif
@@ -1010,7 +993,7 @@ static void add_input_widgets(GtkWidget *dialog, GtkWidget *vbox,
 	{
 		GtkWidget *combo = gtk_combo_box_entry_new_text();
 
-		entry = GTK_BIN(combo)->child;
+		entry = gtk_bin_get_child(GTK_BIN(combo));
 		ui_entry_add_clear_icon(GTK_ENTRY(entry));
 		g_object_set_data(G_OBJECT(dialog), "combo", combo);
 		gtk_container_add(GTK_CONTAINER(vbox), combo);
@@ -1713,5 +1696,3 @@ gint dialogs_show_prompt(GtkWidget *parent,
 	g_free(string);
 	return result;
 }
-
-
