@@ -1,8 +1,8 @@
 /*
  *      document.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: document.c 5904 2011-08-25 20:15:02Z colombanw $
  */
 
 /*
@@ -73,6 +71,9 @@
 #include "win32.h"
 #include "search.h"
 #include "filetypesprivate.h"
+#include "project.h"
+
+#include "SciLexer.h"
 
 
 GeanyFilePrefs file_prefs;
@@ -103,16 +104,9 @@ typedef struct
 
 static void document_undo_clear(GeanyDocument *doc);
 static void document_redo_add(GeanyDocument *doc, guint type, gpointer data);
-static gboolean update_tags_from_buffer(GeanyDocument *doc);
+static gboolean remove_page(guint page_num);
 
 
-/* ignore the case of filenames and paths under WIN32, causes errors if not */
-#ifdef G_OS_WIN32
-#define filenamecmp(a, b)	utils_str_casecmp((a), (b))
-#else
-#define filenamecmp(a, b)	strcmp((a), (b))
-#endif
-
 /**
  * Finds a document whose @c real_path field matches the given filename.
  *
@@ -140,7 +134,7 @@ GeanyDocument* document_find_by_real_path(const gchar *realname)
 		if (! doc->is_valid || ! doc->real_path)
 			continue;
 
-		if (filenamecmp(realname, doc->real_path) == 0)
+		if (utils_filenamecmp(realname, doc->real_path) == 0)
 		{
 			return doc;
 		}
@@ -188,7 +182,7 @@ GeanyDocument *document_find_by_filename(const gchar *utf8_filename)
 		if (! doc->is_valid || doc->file_name == NULL)
 			continue;
 
-		if (filenamecmp(utf8_filename, doc->file_name) == 0)
+		if (utils_filenamecmp(utf8_filename, doc->file_name) == 0)
 		{
 			return doc;
 		}
@@ -280,6 +274,10 @@ void document_init_doclist()
 
 void document_finalize()
 {
+	guint i;
+
+	for (i = 0; i < documents_array->len; i++)
+		g_free(documents[i]);
 	g_ptr_array_free(documents_array, TRUE);
 }
 
@@ -360,42 +358,6 @@ void document_set_text_changed(GeanyDocument *doc, gboolean changed)
 }
 
 
-/* Sets is_valid to FALSE and initializes some members to NULL, to mark it uninitialized.
- * The flag is_valid is set to TRUE in document_create(). */
-static void init_doc_struct(GeanyDocument *new_doc)
-{
-	GeanyDocumentPrivate *priv;
-
-	memset(new_doc, 0, sizeof(GeanyDocument));
-
-	new_doc->is_valid = FALSE;
-	new_doc->has_tags = FALSE;
-	new_doc->readonly = FALSE;
-	new_doc->file_name = NULL;
-	new_doc->file_type = NULL;
-	new_doc->tm_file = NULL;
-	new_doc->encoding = NULL;
-	new_doc->has_bom = FALSE;
-	new_doc->editor = NULL;
-	new_doc->changed = FALSE;
-	new_doc->real_path = NULL;
-
-	new_doc->priv = g_new0(GeanyDocumentPrivate, 1);
-	priv = new_doc->priv;
-	priv->tag_store = NULL;
-	priv->tag_tree = NULL;
-	priv->saved_encoding.encoding = NULL;
-	priv->saved_encoding.has_bom = FALSE;
-	priv->undo_actions = NULL;
-	priv->redo_actions = NULL;
-	priv->line_count = 0;
-	priv->tag_list_update_source = 0;
-#ifndef USE_GIO_FILEMON
-	priv->last_check = time(NULL);
-#endif
-}
-
-
 /* returns the next free place in the document list,
  * or -1 if the documents_array is full */
 static gint document_get_new_idx(void)
@@ -536,8 +498,6 @@ static gboolean on_idle_focus(gpointer doc)
 }
 
 
-static gboolean remove_page(guint page_num);
-
 /* Creates a new document and editor, adding a tab in the notebook.
  * @return The created document */
 static GeanyDocument *document_create(const gchar *utf8_filename)
@@ -548,9 +508,9 @@ static GeanyDocument *document_create(const gchar *utf8_filename)
 
 	if (cur_pages == 1)
 	{
-		GeanyDocument *cur = document_get_current();
+		doc = document_get_current();
 		/* remove the empty document first */
-		if (cur != NULL && cur->file_name == NULL && ! cur->changed)
+		if (doc != NULL && doc->file_name == NULL && ! doc->changed)
 			/* prevent immediately opening another new doc with
 			 * new_document_after_close pref */
 			remove_page(0);
@@ -559,18 +519,22 @@ static GeanyDocument *document_create(const gchar *utf8_filename)
 	new_idx = document_get_new_idx();
 	if (new_idx == -1)	/* expand the array, no free places */
 	{
-		GeanyDocument *new_doc = g_new0(GeanyDocument, 1);
+		doc = g_new0(GeanyDocument, 1);
 
 		new_idx = documents_array->len;
-		g_ptr_array_add(documents_array, new_doc);
+		g_ptr_array_add(documents_array, doc);
 	}
+
 	doc = documents[new_idx];
-	init_doc_struct(doc);	/* initialize default document settings */
+
+	/* initialize default document settings */
+	doc->priv = g_new0(GeanyDocumentPrivate, 1);
 	doc->index = new_idx;
-
 	doc->file_name = g_strdup(utf8_filename);
-
 	doc->editor = editor_create(doc);
+#ifndef USE_GIO_FILEMON
+	doc->priv->last_check = time(NULL);
+#endif
 
 	sidebar_openfiles_add(doc);	/* sets doc->iter */
 
@@ -608,28 +572,23 @@ gboolean document_close(GeanyDocument *doc)
 }
 
 
-/* Call document_remove_page() instead, this is only needed for document_create(). */
+/* Call document_remove_page() instead, this is only needed for document_create()
+ * to prevent re-opening a new document when the last document is closed (if enabled). */
 static gboolean remove_page(guint page_num)
 {
 	GeanyDocument *doc = document_get_from_page(page_num);
 
-	if (G_UNLIKELY(doc == NULL))
-	{
-		g_warning("%s: page_num: %d", G_STRFUNC, page_num);
-		return FALSE;
-	}
+	g_return_val_if_fail(doc != NULL, FALSE);
 
 	if (doc->changed && ! dialogs_show_unsaved_file(doc))
-	{
 		return FALSE;
-	}
 
 	/* tell any plugins that the document is about to be closed */
 	g_signal_emit_by_name(geany_object, "document-close", doc);
 
 	/* Checking real_path makes it likely the file exists on disk */
 	if (! main_status.closing_all && doc->real_path != NULL)
-		ui_add_recent_file(doc->file_name);
+		ui_add_recent_document(doc);
 
 	doc->is_valid = FALSE;
 
@@ -647,23 +606,20 @@ static gboolean remove_page(guint page_num)
 	tm_workspace_remove_object(doc->tm_file, TRUE, TRUE);
 
 	editor_destroy(doc->editor);
-	doc->editor = NULL;
+	doc->editor = NULL; /* needs to be NULL for document_undo_clear() call below */
 
 	document_stop_file_monitoring(doc);
 
-	doc->file_name = NULL;
-	doc->real_path = NULL;
-	doc->file_type = NULL;
-	doc->encoding = NULL;
-	doc->has_bom = FALSE;
-	doc->tm_file = NULL;
 	document_undo_clear(doc);
+
 	g_free(doc->priv);
 
+	/* reset document settings to defaults for re-use */
+	memset(doc, 0, sizeof(GeanyDocument));
+
 	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)) == 0)
 	{
 		sidebar_update_tag_list(NULL, FALSE);
-		/*on_notebook1_switch_page(GTK_NOTEBOOK(main_widgets.notebook), NULL, 0, NULL);*/
 		ui_set_window_title(NULL);
 		ui_save_buttons_toggle(FALSE);
 		ui_update_popup_reundo_items(NULL);
@@ -762,11 +718,10 @@ GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft,
 	if (ft == NULL && utf8_filename != NULL) /* guess the filetype from the filename if one is given */
 		ft = filetypes_detect_from_document(doc);
 
-	document_set_filetype(doc, ft);	/* also clears taglist */
+	document_set_filetype(doc, ft); /* also re-parses tags */
 
 	ui_set_window_title(doc);
 	build_menu_update(doc);
-	document_update_tag_list(doc, FALSE);
 	document_set_text_changed(doc, FALSE);
 	ui_document_show_hide(doc); /* update the document menu */
 
@@ -1108,6 +1063,13 @@ void document_apply_indent_settings(GeanyDocument *doc)
 }
 
 
+void document_show_tab(GeanyDocument *doc)
+{
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
+		document_get_notebook_page(doc));
+}
+
+
 /* To open a new file, set doc to NULL; filename should be locale encoded.
  * To reload a file, set the doc for the document to be reloaded; filename should be NULL.
  * pos is the cursor position, which can be overridden by --line and --column.
@@ -1132,11 +1094,7 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 	else
 	{
 		/* filename must not be NULL when opening a file */
-		if (filename == NULL)
-		{
-			ui_set_statusbar(FALSE, _("Invalid filename"));
-			return NULL;
-		}
+		g_return_val_if_fail(filename, NULL);
 
 #ifdef G_OS_WIN32
 		/* if filename is a shortcut, try to resolve it */
@@ -1154,11 +1112,9 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 		doc = document_find_by_filename(utf8_filename);
 		if (doc != NULL)
 		{
-			ui_add_recent_file(utf8_filename);	/* either add or reorder recent item */
+			ui_add_recent_document(doc);	/* either add or reorder recent item */
 			/* show the doc before reload dialog */
-			gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-				gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
-				(GtkWidget*) doc->editor->sci));
+			document_show_tab(doc);
 			document_check_disk_status(doc, TRUE);	/* force a file changed check */
 		}
 	}
@@ -1180,7 +1136,7 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 			g_return_val_if_fail(doc != NULL, NULL); /* really should not happen */
 
 			/* file exists on disk, set real_path */
-			setptr(doc->real_path, tm_get_real_path(locale_filename));
+			SETPTR(doc->real_path, tm_get_real_path(locale_filename));
 
 			doc->priv->is_remote = utils_is_remote_path(locale_filename);
 			monitor_file_setup(doc);
@@ -1243,7 +1199,7 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 
 		/* finally add current file to recent files menu, but not the files from the last session */
 		if (! main_status.opening_session_files)
-			ui_add_recent_file(utf8_filename);
+			ui_add_recent_document(doc);
 
 		if (reload)
 		{
@@ -1290,10 +1246,11 @@ void document_open_file_list(const gchar *data, gsize length)
 
 	list = g_strsplit(data, utils_get_eol_char(utils_get_line_endings(data, length)), 0);
 
-	for (i = 0; list[i] != NULL; i++)
+	/* stop at the end or first empty item, because last item is empty but not null */
+	for (i = 0; list[i] != NULL && list[i][0] != '\0'; i++)
 	{
-		filename = g_filename_from_uri(list[i], NULL, NULL);
-		if (G_UNLIKELY(filename == NULL))
+		filename = utils_get_path_from_uri(list[i]);
+		if (filename == NULL)
 			continue;
 		document_open_file(filename, FALSE, NULL, NULL);
 		g_free(filename);
@@ -1490,10 +1447,10 @@ gboolean document_save_file_as(GeanyDocument *doc, const gchar *utf8_fname)
 	g_return_val_if_fail(doc != NULL, FALSE);
 
 	if (utf8_fname != NULL)
-		setptr(doc->file_name, g_strdup(utf8_fname));
+		SETPTR(doc->file_name, g_strdup(utf8_fname));
 
 	/* reset real path, it's retrieved again in document_save() */
-	setptr(doc->real_path, NULL);
+	SETPTR(doc->real_path, NULL);
 
 	/* detect filetype */
 	if (doc->file_type->id == GEANY_FILETYPES_NONE)
@@ -1518,7 +1475,7 @@ gboolean document_save_file_as(GeanyDocument *doc, const gchar *utf8_fname)
 	doc->priv->file_disk_status = FILE_IGNORE;
 
 	if (ret)
-		ui_add_recent_file(doc->file_name);
+		ui_add_recent_document(doc);
 	return ret;
 }
 
@@ -1530,7 +1487,7 @@ static gsize save_convert_to_encoding(GeanyDocument *doc, gchar **data, gsize *l
 	gsize bytes_read;
 	gsize conv_len;
 
-	g_return_val_if_fail(data != NULL || *data == NULL, FALSE);
+	g_return_val_if_fail(data != NULL && *data != NULL, FALSE);
 	g_return_val_if_fail(len != NULL, FALSE);
 
 	/* try to convert it from UTF-8 to original encoding */
@@ -1587,7 +1544,7 @@ _("An error occurred while converting the file from UTF-8 in \"%s\". The file re
 
 
 static gchar *write_data_to_disk(const gchar *locale_filename,
-								 const gchar *data, gint len)
+								 const gchar *data, gsize len)
 {
 	GError *error = NULL;
 
@@ -1633,7 +1590,7 @@ static gchar *write_data_to_disk(const gchar *locale_filename,
 		}
 		else
 		{
-			gint bytes_written;
+			gsize bytes_written;
 
 			errno = 0;
 			bytes_written = fwrite(data, sizeof(gchar), len, fp);
@@ -1679,7 +1636,7 @@ static gchar *write_data_to_disk(const gchar *locale_filename,
 
 
 static gchar *save_doc(GeanyDocument *doc, const gchar *locale_filename,
-								 const gchar *data, gint len)
+								 const gchar *data, gsize len)
 {
 	gchar *err;
 
@@ -1702,20 +1659,27 @@ static gchar *save_doc(GeanyDocument *doc, const gchar *locale_filename,
 
 
 /**
- *  Saves the document. Saving may include replacing tabs by spaces,
+ *  Saves the document.
+ *  Also shows the Save As dialog if necessary.
+ *  If the file is not modified, this function may do nothing unless @a force is set to @c TRUE.
+ *
+ *  Saving may include replacing tabs by spaces,
  *  stripping trailing spaces and adding a final new line at the end of the file, depending
  *  on user preferences. Then the @c "document-before-save" signal is emitted,
  *  allowing plugins to modify the document before it is saved, and data is
- *  actually written to disk. The filetype is set again or auto-detected if it wasn't set yet.
- *  Afterwards, the @c "document-save" signal is emitted for plugins.
+ *  actually written to disk.
  *
- *  If the file is not modified, this functions does nothing unless force is set to @c TRUE.
+ *  On successful saving:
+ *  - GeanyDocument::real_path is set.
+ *  - The filetype is set again or auto-detected if it wasn't set yet.
+ *  - The @c "document-save" signal is emitted for plugins.
  *
- *  @note You should ensure @c doc->file_name is not @c NULL before calling this; otherwise
- *  call dialogs_show_save_as().
+ *  @warning You should ensure @c doc->file_name has an absolute path unless you want the
+ *  Save As dialog to be shown. A @c NULL value also shows the dialog. This behaviour was
+ *  added in Geany 1.22.
  *
  *  @param doc The document to save.
- *  @param force Whether to save the file even if it is not modified (e.g. for Save As).
+ *  @param force Whether to save the file even if it is not modified.
  *
  *  @return @c TRUE if the file was saved or @c FALSE if the file could not or should not be saved.
  **/
@@ -1725,31 +1689,33 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 	gchar *data;
 	gsize len;
 	gchar *locale_filename;
+	const GeanyFilePrefs *fp;
 
 	g_return_val_if_fail(doc != NULL, FALSE);
 
+	if (document_need_save_as(doc))
+	{
+		/* ensure doc is the current tab before showing the dialog */
+		document_show_tab(doc);
+		return dialogs_show_save_as();
+	}
+
 	/* the "changed" flag should exclude the "readonly" flag, but check it anyway for safety */
 	if (! force && ! ui_prefs.allow_always_save && (! doc->changed || doc->readonly))
 		return FALSE;
 
-	if (G_UNLIKELY(doc->file_name == NULL))
-	{
-		ui_set_statusbar(TRUE, _("Error saving file (%s)."), _("Invalid filename"));
-		utils_beep();
-		return FALSE;
-	}
-
+	fp = project_get_file_prefs();
 	/* replaces tabs by spaces but only if the current file is not a Makefile */
-	if (file_prefs.replace_tabs && doc->file_type->id != GEANY_FILETYPES_MAKE)
+	if (fp->replace_tabs && doc->file_type->id != GEANY_FILETYPES_MAKE)
 		editor_replace_tabs(doc->editor);
 	/* strip trailing spaces */
-	if (file_prefs.strip_trailing_spaces)
+	if (fp->strip_trailing_spaces)
 		editor_strip_trailing_spaces(doc->editor);
 	/* ensure the file has a newline at the end */
-	if (file_prefs.final_new_line)
+	if (fp->final_new_line)
 		editor_ensure_final_newline(doc->editor);
 	/* ensure newlines are consistent */
-	if (file_prefs.ensure_convert_new_lines)
+	if (fp->ensure_convert_new_lines)
 		sci_convert_eols(doc->editor->sci, sci_get_eol_mode(doc->editor->sci));
 
 	/* notify plugins which may wish to modify the document before it's saved */
@@ -1803,7 +1769,7 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 
 		if (!file_prefs.use_safe_file_saving)
 		{
-			setptr(errmsg,
+			SETPTR(errmsg,
 				g_strdup_printf(_("%s\n\nThe file on disk may now be truncated!"), errmsg));
 		}
 		dialogs_show_msgbox_with_secondary(GTK_MESSAGE_ERROR, _("Error saving file."), errmsg);
@@ -1977,7 +1943,7 @@ gint document_find_text(GeanyDocument *doc, const gchar *text, const gchar *orig
 		}
 
 		/* we searched only part of the document, so ask whether to wraparound. */
-		if (search_prefs.suppress_dialogs ||
+		if (search_prefs.always_wrap ||
 			dialogs_show_question_full(parent, GTK_STOCK_FIND, GTK_STOCK_CANCEL,
 				_("Wrap search and find again?"), _("\"%s\" was not found."), original_text))
 		{
@@ -2243,43 +2209,32 @@ gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gcha
 }
 
 
-static gboolean update_tags_from_buffer(GeanyDocument *doc)
+/*
+ * Parses or re-parses the document's buffer and updates the type
+ * keywords and symbol list.
+ *
+ * @param doc The document.
+ */
+void document_update_tags(GeanyDocument *doc)
 {
-	gboolean result;
-#if 0
-		/* old code */
-		result = tm_source_file_update(doc->tm_file, TRUE, FALSE, TRUE);
-#else
-		gsize len = sci_get_length(doc->editor->sci) + 1;
-		gchar *text = g_malloc(len);
+	guchar *buffer_ptr;
+	gsize len;
 
-		/* we copy the whole text into memory instead using a direct char pointer from
-		 * Scintilla because tm_source_file_buffer_update() does modify the string slightly */
-		sci_get_text(doc->editor->sci, len, text);
-		result = tm_source_file_buffer_update(doc->tm_file, (guchar*) text, len, TRUE);
-		g_free(text);
-#endif
-	return result;
-}
+	g_return_if_fail(DOC_VALID(doc));
+	g_return_if_fail(app->tm_workspace != NULL);
 
-
-void document_update_tag_list(GeanyDocument *doc, gboolean update)
-{
-	/* We must call sidebar_update_tag_list() before returning,
-	 * to ensure that the symbol list is always updated properly (e.g.
-	 * when creating a new document with a partial filename set. */
-	gboolean success = FALSE;
-
-	/* if the filetype doesn't have a tag parser or it is a new file */
-	if (doc == NULL || doc->file_type == NULL || app->tm_workspace == NULL ||
-		! filetype_has_tags(doc->file_type) || ! doc->file_name)
+	/* early out if it's a new file or doesn't support tags */
+	if (! doc->file_name || ! doc->file_type || !filetype_has_tags(doc->file_type))
 	{
-		/* set the default (empty) tag list */
+		/* We must call sidebar_update_tag_list() before returning,
+		 * to ensure that the symbol list is always updated properly (e.g.
+		 * when creating a new document with a partial filename set. */
 		sidebar_update_tag_list(doc, FALSE);
 		return;
 	}
 
-	if (doc->tm_file == NULL)
+	/* create a new TM file if there isn't one yet */
+	if (! doc->tm_file)
 	{
 		gchar *locale_filename = utils_get_locale_from_utf8(doc->file_name);
 		const gchar *name;
@@ -2289,28 +2244,88 @@ void document_update_tag_list(GeanyDocument *doc, gboolean update)
 		doc->tm_file = tm_source_file_new(locale_filename, FALSE, name);
 		g_free(locale_filename);
 
-		if (doc->tm_file)
+		if (doc->tm_file && !tm_workspace_add_object(doc->tm_file))
 		{
-			if (!tm_workspace_add_object(doc->tm_file))
-			{
-				tm_work_object_free(doc->tm_file);
-				doc->tm_file = NULL;
-			}
-			else
-			{
-				if (update)
-					update_tags_from_buffer(doc);
-				success = TRUE;
-			}
+			tm_work_object_free(doc->tm_file);
+			doc->tm_file = NULL;
 		}
 	}
-	else
+
+	/* early out if there's no work object and we couldn't create one */
+	if (doc->tm_file == NULL)
 	{
-		success = update_tags_from_buffer(doc);
-		if (G_UNLIKELY(! success))
-			geany_debug("tag list updating failed");
+		/* We must call sidebar_update_tag_list() before returning,
+		 * to ensure that the symbol list is always updated properly (e.g.
+		 * when creating a new document with a partial filename set. */
+		sidebar_update_tag_list(doc, FALSE);
+		return;
+	}
+
+	len = sci_get_length(doc->editor->sci);
+	/* tm_source_file_buffer_update() below don't support 0-length data,
+	 * so just empty the tags array and leave */
+	if (len < 1)
+	{
+		tm_tags_array_free(doc->tm_file->tags_array, FALSE);
+		sidebar_update_tag_list(doc, FALSE);
+		return;
+	}
+
+	/* Parse Scintilla's buffer directly using TagManager
+	 * Note: this buffer *MUST NOT* be modified */
+	buffer_ptr = (guchar *) scintilla_send_message(doc->editor->sci, SCI_GETCHARACTERPOINTER, 0, 0);
+	tm_source_file_buffer_update(doc->tm_file, buffer_ptr, len, TRUE);
+
+	sidebar_update_tag_list(doc, TRUE);
+	document_highlight_tags(doc);
+}
+
+
+/* Re-highlights type keywords without re-parsing the whole document. */
+void document_highlight_tags(GeanyDocument *doc)
+{
+	GString *keywords_str;
+	gchar *keywords;
+	gint keyword_idx;
+
+	/* some filetypes support type keywords (such as struct names), but not
+	 * necessarily all filetypes for a particular scintilla lexer.  this
+	 * tells us whether the filetype supports keywords, and if so
+	 * which index to use for the scintilla keywords set. */
+	switch (doc->file_type->id)
+	{
+		case GEANY_FILETYPES_C:
+		case GEANY_FILETYPES_CPP:
+		case GEANY_FILETYPES_CS:
+		case GEANY_FILETYPES_D:
+		case GEANY_FILETYPES_JAVA:
+		case GEANY_FILETYPES_OBJECTIVEC:
+		case GEANY_FILETYPES_VALA:
+		{
+
+			/* index of the keyword set in the Scintilla lexer, for
+			 * example in LexCPP.cxx, see "cppWordLists" global array.
+			 * TODO: this magic number should be a member of the filetype */
+			keyword_idx = 3;
+			break;
+		}
+		default:
+			return; /* early out if type keywords are not supported */
+	}
+	if (!app->tm_workspace->work_object.tags_array)
+		return;
+
+	/* get any type keywords and tell scintilla about them
+	 * this will cause the type keywords to be colourized in scintilla */
+	keywords_str = symbols_find_tags_as_string(app->tm_workspace->work_object.tags_array,
+		TM_GLOBAL_TYPE_MASK, doc->file_type->lang);
+	if (keywords_str)
+	{
+		keywords = g_string_free(keywords_str, FALSE);
+		sci_set_keywords(doc->editor->sci, keyword_idx, keywords);
+		g_free(keywords);
+		queue_colourise(doc); /* force re-highlighting the entire document */
 	}
-	sidebar_update_tag_list(doc, success);
 }
 
 
@@ -2322,9 +2337,11 @@ static gboolean on_document_update_tag_list_idle(gpointer data)
 		return FALSE;
 
 	if (! main_status.quitting)
-		document_update_tag_list(doc, TRUE);
+		document_update_tags(doc);
 
 	doc->priv->tag_list_update_source = 0;
+
+	/* don't update the tags until another modification of the buffer */
 	return FALSE;
 }
 
@@ -2334,113 +2351,15 @@ void document_update_tag_list_in_idle(GeanyDocument *doc)
 	if (editor_prefs.autocompletion_update_freq <= 0 || ! filetype_has_tags(doc->file_type))
 		return;
 
+	/* prevent "stacking up" callback handlers, we only need one to run soon */
 	if (doc->priv->tag_list_update_source != 0)
 		g_source_remove(doc->priv->tag_list_update_source);
+
 	doc->priv->tag_list_update_source = g_timeout_add_full(G_PRIORITY_LOW,
 		editor_prefs.autocompletion_update_freq, on_document_update_tag_list_idle, doc, NULL);
 }
 
 
-/* Caches the list of project typenames, as a space separated GString.
- * Returns: TRUE if typenames have changed.
- * (*types) is set to the list of typenames, or NULL if there are none. */
-static gboolean get_project_typenames(const GString **types, gint lang)
-{
-	static GString *last_typenames = NULL;
-	GString *s = NULL;
-
-	if (app->tm_workspace)
-	{
-		GPtrArray *tags_array = app->tm_workspace->work_object.tags_array;
-
-		if (tags_array)
-		{
-			s = symbols_find_tags_as_string(tags_array, TM_GLOBAL_TYPE_MASK, lang);
-		}
-	}
-
-	if (s && last_typenames && g_string_equal(s, last_typenames))
-	{
-		g_string_free(s, TRUE);
-		*types = last_typenames;
-		return FALSE;	/* project typenames haven't changed */
-	}
-	/* cache typename list for next time */
-	if (last_typenames)
-		g_string_free(last_typenames, TRUE);
-	last_typenames = s;
-
-	*types = s;
-	if (s == NULL)
-		return FALSE;
-	return TRUE;
-}
-
-
-/* If sci is NULL, update project typenames for all documents that support typenames,
- * if typenames have changed.
- * If sci is not NULL, then if sci supports typenames, project typenames are updated
- * if necessary, and typename keywords are set for sci.
- * Returns: TRUE if any scintilla type keywords were updated. */
-static gboolean update_type_keywords(GeanyDocument *doc, gint lang)
-{
-	gboolean ret = FALSE;
-	guint n;
-	const GString *s;
-	ScintillaObject *sci;
-
-	g_return_val_if_fail(doc != NULL, FALSE);
-	sci = doc->editor->sci;
-
-	switch (doc->file_type->id)
-	{	/* continue working with the following languages, skip on all others */
-		case GEANY_FILETYPES_C:
-		case GEANY_FILETYPES_CPP:
-		case GEANY_FILETYPES_CS:
-		case GEANY_FILETYPES_D:
-		case GEANY_FILETYPES_JAVA:
-		case GEANY_FILETYPES_VALA:
-			break;
-		default:
-			return FALSE;
-	}
-
-	sci = doc->editor->sci;
-	if (sci != NULL && editor_lexer_get_type_keyword_idx(sci_get_lexer(sci)) == -1)
-		return FALSE;
-
-	if (! get_project_typenames(&s, lang))
-	{	/* typenames have not changed */
-		if (s != NULL && sci != NULL)
-		{
-			gint keyword_idx = editor_lexer_get_type_keyword_idx(sci_get_lexer(sci));
-
-			sci_set_keywords(sci, keyword_idx, s->str);
-			queue_colourise(doc);
-		}
-		return FALSE;
-	}
-	g_return_val_if_fail(s != NULL, FALSE);
-
-	for (n = 0; n < documents_array->len; n++)
-	{
-		if (documents[n]->is_valid)
-		{
-			ScintillaObject *wid = documents[n]->editor->sci;
-			gint keyword_idx = editor_lexer_get_type_keyword_idx(sci_get_lexer(wid));
-
-			if (keyword_idx > 0)
-			{
-				sci_set_keywords(wid, keyword_idx, s->str);
-				queue_colourise(documents[n]);
-				ret = TRUE;
-			}
-		}
-	}
-	return ret;
-}
-
-
 static void document_load_config(GeanyDocument *doc, GeanyFiletype *type,
 		gboolean filetype_changed)
 {
@@ -2469,10 +2388,7 @@ static void document_load_config(GeanyDocument *doc, GeanyFiletype *type,
 		doc->priv->symbol_list_sort_mode = type->priv->symbol_list_sort_mode;
 	}
 
-	document_update_tag_list(doc, TRUE);
-
-	/* Update session typename keywords. */
-	update_type_keywords(doc, type->lang);
+	document_update_tags(doc);
 }
 
 
@@ -2854,13 +2770,14 @@ GeanyDocument *document_clone(GeanyDocument *old_doc, const gchar *utf8_filename
  * @return TRUE if all files were saved or had their changes discarded. */
 gboolean document_account_for_unsaved(void)
 {
-	guint i, p, page_count, len = documents_array->len;
-	GeanyDocument *doc;
+	guint i, p, page_count;
 
 	page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
+	/* iterate over documents in tabs order */
 	for (p = 0; p < page_count; p++)
 	{
-		doc = document_get_from_page(p);
+		GeanyDocument *doc = document_get_from_page(p);
+
 		if (DOC_VALID(doc) && doc->changed)
 		{
 			if (! dialogs_show_unsaved_file(doc))
@@ -2868,13 +2785,9 @@ gboolean document_account_for_unsaved(void)
 		}
 	}
 	/* all documents should now be accounted for, so ignore any changes */
-	for (i = 0; i < len; i++)
+	foreach_document (i)
 	{
-		doc = documents[i];
-		if (doc->is_valid && doc->changed)
-		{
-			doc->changed = FALSE;
-		}
+		documents[i]->changed = FALSE;
 	}
 	return TRUE;
 }
@@ -2962,7 +2875,7 @@ static gboolean monitor_resave_missing_file(GeanyDocument *doc)
 		/* file is missing - set unsaved state */
 		document_set_text_changed(doc, TRUE);
 		/* don't prompt more than once */
-		setptr(doc->real_path, NULL);
+		SETPTR(doc->real_path, NULL);
 	}
 
 	return want_reload;
@@ -2984,7 +2897,8 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 	g_return_val_if_fail(doc != NULL, FALSE);
 
 	/* ignore remote files and documents that have never been saved to disk */
-	if (file_prefs.disk_check_timeout == 0 || doc->real_path == NULL || doc->priv->is_remote)
+	if (notebook_switch_in_progress() || file_prefs.disk_check_timeout == 0
+			|| doc->real_path == NULL || doc->priv->is_remote)
 		return FALSE;
 
 	use_gio_filemon = (doc->priv->monitor != NULL);
@@ -3010,10 +2924,11 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 		/* doc may be closed now */
 		ret = TRUE;
 	}
-	else if (G_UNLIKELY(! use_gio_filemon && /* ignore these checks when using GIO */
-			 (doc->priv->mtime > cur_time || st.st_mtime > cur_time)))
+	else if (! use_gio_filemon && /* ignore check when using GIO */
+		doc->priv->mtime > cur_time)
 	{
 		g_warning("%s: Something is wrong with the time stamps.", G_STRFUNC);
+		/* Note: on Windows st.st_mtime can be newer than cur_time */
 	}
 	else if (doc->priv->mtime < st.st_mtime)
 	{
