@@ -1,8 +1,8 @@
 /*
  *      document.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  $Id: document.h 5904 2011-08-25 20:15:02Z colombanw $
  */
 
 /**
@@ -31,6 +29,8 @@
 #ifndef GEANY_DOCUMENT_H
 #define GEANY_DOCUMENT_H 1
 
+G_BEGIN_DECLS
+
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 #include "editor.h"
@@ -63,6 +63,8 @@ typedef struct GeanyFilePrefs
 	gboolean		ensure_convert_new_lines;
 	gboolean		gio_unsafe_save_backup;
 	gboolean		use_gio_unsafe_file_saving; /* whether to use GIO as the unsafe backend */
+	gchar			*extract_filetype_regex;	/* regex to extract filetype on opening */
+	gboolean		tab_close_switch_to_mru;
 }
 GeanyFilePrefs;
 
@@ -192,6 +194,8 @@ GeanyDocument* document_get_from_page(guint page_num);
 
 GeanyDocument *document_get_current(void);
 
+void document_show_tab(GeanyDocument *doc);
+
 void document_init_doclist(void);
 
 void document_finalize(void);
@@ -231,10 +235,12 @@ gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gcha
 void document_replace_sel(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
 						  const gchar *original_find_text, const gchar *original_replace_text, gint flags);
 
-void document_update_tag_list(GeanyDocument *doc, gboolean update);
+void document_update_tags(GeanyDocument *doc);
 
 void document_update_tag_list_in_idle(GeanyDocument *doc);
 
+void document_highlight_tags(GeanyDocument *doc);
+
 void document_set_encoding(GeanyDocument *doc, const gchar *new_encoding);
 
 gboolean document_check_disk_status(GeanyDocument *doc, gboolean force);
@@ -275,4 +281,6 @@ gint document_compare_by_tab_order_reverse(gconstpointer a, gconstpointer b);
 
 void document_grab_focus(GeanyDocument *doc);
 
+G_END_DECLS
+
 #endif
