@@ -1,8 +1,8 @@
 /*
  *      encodings.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  $Id: encodings.h 2267 2008-02-20 11:24:23Z eht16 $
  */
 
 
@@ -38,6 +36,7 @@
 #ifndef GEANY_ENCODINGS_H
 #define GEANY_ENCODINGS_H
 
+G_BEGIN_DECLS
 
 typedef enum
 {
@@ -63,9 +62,9 @@ typedef struct
 	/** Internally used member for grouping */
 	GeanyEncodingGroup   group;
 	/** String representation of the encoding, e.g. "ISO-8859-3" */
-	gchar				*charset;
+	const gchar			*charset;
 	/** Translatable and descriptive name of the encoding, e.g. "South European" */
-	gchar				*name;
+	const gchar			*name;
 } GeanyEncoding;
 
 
@@ -81,15 +80,17 @@ void encodings_select_radio_item(const gchar *charset);
 void encodings_init(void);
 void encodings_finalize(void);
 
-gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_encoding);
+gchar *encodings_convert_to_utf8(const gchar *buffer, gssize size, gchar **used_encoding);
 
 /* Converts a string from the given charset to UTF-8.
  * If fast is set, no further checks are performed. */
-gchar *encodings_convert_to_utf8_from_charset(const gchar *buffer, gsize size,
+gchar *encodings_convert_to_utf8_from_charset(const gchar *buffer, gssize size,
 											  const gchar *charset, gboolean fast);
 
 gboolean encodings_is_unicode_charset(const gchar *string);
 
+gboolean encodings_convert_to_utf8_auto(gchar **buf, gsize *size, const gchar *forced_enc,
+		gchar **used_encoding, gboolean *has_bom, gboolean *partial);
 
 /*
  * The original versions of the following tables are taken from profterm
@@ -174,6 +175,7 @@ typedef enum
 	GEANY_ENCODING_WINDOWS_1258,
 
 	GEANY_ENCODING_NONE,
+	GEANY_ENCODING_CP_932,
 
 	GEANY_ENCODINGS_MAX
 } GeanyEncodingIndex;
@@ -186,4 +188,6 @@ GeanyEncodingIndex encodings_scan_unicode_bom(const gchar *string, gsize len, gu
 
 GeanyEncodingIndex encodings_get_idx_from_charset(const gchar *charset);
 
+G_END_DECLS
+
 #endif
