@@ -1,8 +1,8 @@
 /*
  *      geanywraplabel.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  $Id: geanywraplabel.c 5795 2011-05-10 23:51:01Z colombanw $
  */
 
 /*
@@ -33,9 +31,6 @@
 
 
 
-#define GEANY_WRAP_LABEL_GET_PRIVATE(obj)	(GEANY_WRAP_LABEL(obj)->priv)
-
-
 struct _GeanyWrapLabelClass
 {
 	GtkLabelClass parent_class;
@@ -43,7 +38,8 @@ struct _GeanyWrapLabelClass
 
 typedef struct
 {
-	gsize wrap_width;
+	gint wrap_width;
+	gint wrap_height;
 } GeanyWrapLabelPrivate;
 
 struct _GeanyWrapLabel
@@ -55,7 +51,8 @@ struct _GeanyWrapLabel
 
 static void geany_wrap_label_size_request	(GtkWidget *widget, GtkRequisition *req);
 static void geany_wrap_label_size_allocate	(GtkWidget *widget, GtkAllocation *alloc);
-static void geany_wrap_label_set_wrap_width	(GtkWidget *widget, gsize width);
+static gboolean geany_wrap_label_expose		(GtkWidget *widget, GdkEventExpose *event);
+static void geany_wrap_label_set_wrap_width	(GtkWidget *widget, gint width);
 static void geany_wrap_label_label_notify	(GObject *object, GParamSpec *pspec, gpointer data);
 
 G_DEFINE_TYPE(GeanyWrapLabel, geany_wrap_label, GTK_TYPE_LABEL)
@@ -67,6 +64,7 @@ static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
 
 	widget_class->size_request = geany_wrap_label_size_request;
 	widget_class->size_allocate = geany_wrap_label_size_allocate;
+	widget_class->expose_event = geany_wrap_label_expose;
 
 	g_type_class_add_private(klass, sizeof (GeanyWrapLabelPrivate));
 }
@@ -74,38 +72,39 @@ static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
 
 static void geany_wrap_label_init(GeanyWrapLabel *self)
 {
-	GeanyWrapLabelPrivate *priv;
-
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self,
 		GEANY_WRAP_LABEL_TYPE, GeanyWrapLabelPrivate);
 
-	priv = self->priv;
-	priv->wrap_width = 0;
+	self->priv->wrap_width = 0;
+	self->priv->wrap_height = 0;
 
 	g_signal_connect(self, "notify::label", G_CALLBACK(geany_wrap_label_label_notify), NULL);
-	pango_layout_set_wrap(gtk_label_get_layout(GTK_LABEL(self)), PANGO_WRAP_WORD_CHAR);
 	gtk_misc_set_alignment(GTK_MISC(self), 0.0, 0.0);
 }
 
 
 /* Sets the point at which the text should wrap. */
-static void geany_wrap_label_set_wrap_width(GtkWidget *widget, gsize width)
+static void geany_wrap_label_set_wrap_width(GtkWidget *widget, gint width)
 {
-	GeanyWrapLabelPrivate *priv;
+	GeanyWrapLabel *self = GEANY_WRAP_LABEL(widget);
+	PangoLayout *layout;
 
-	if (width == 0)
+	if (width <= 0)
 		return;
 
+	layout = gtk_label_get_layout(GTK_LABEL(widget));
+
 	/*
 	* We may need to reset the wrap width, so do this regardless of whether
 	* or not we've changed the width.
 	*/
-	pango_layout_set_width(gtk_label_get_layout(GTK_LABEL(widget)), width * PANGO_SCALE);
+	pango_layout_set_width(layout, width * PANGO_SCALE);
+	pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
+	pango_layout_get_pixel_size(layout, NULL, &self->priv->wrap_height);
 
-	priv = GEANY_WRAP_LABEL_GET_PRIVATE(widget);
-	if (priv->wrap_width != width)
+	if (self->priv->wrap_width != width)
 	{
-		priv->wrap_width = width;
+		self->priv->wrap_width = width;
 		gtk_widget_queue_resize(widget);
 	}
 }
@@ -114,9 +113,22 @@ static void geany_wrap_label_set_wrap_width(GtkWidget *widget, gsize width)
 /* updates the wrap width when the label text changes */
 static void geany_wrap_label_label_notify(GObject *object, GParamSpec *pspec, gpointer data)
 {
-	GeanyWrapLabelPrivate *priv = GEANY_WRAP_LABEL_GET_PRIVATE(object);
+	GeanyWrapLabel *self = GEANY_WRAP_LABEL(object);
 
-	geany_wrap_label_set_wrap_width(GTK_WIDGET(object), priv->wrap_width);
+	geany_wrap_label_set_wrap_width(GTK_WIDGET(object), self->priv->wrap_width);
+}
+
+
+/* makes sure the layout is setup for rendering and chains to parent renderer */
+static gboolean geany_wrap_label_expose(GtkWidget *widget, GdkEventExpose *event)
+{
+	GeanyWrapLabel *self = GEANY_WRAP_LABEL(widget);
+	PangoLayout *layout = gtk_label_get_layout(GTK_LABEL(widget));
+
+	pango_layout_set_width(layout, self->priv->wrap_width * PANGO_SCALE);
+	pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
+
+	return (* GTK_WIDGET_CLASS(geany_wrap_label_parent_class)->expose_event)(widget, event);
 }
 
 
@@ -124,12 +136,8 @@ static void geany_wrap_label_label_notify(GObject *object, GParamSpec *pspec, gp
  * width to be flexible. */
 static void geany_wrap_label_size_request(GtkWidget *widget, GtkRequisition *req)
 {
-	gint height;
-
-	pango_layout_get_pixel_size(gtk_label_get_layout(GTK_LABEL(widget)), NULL, &height);
-
-	req->width  = 0;
-	req->height = height;
+	req->width = 0;
+	req->height = GEANY_WRAP_LABEL(widget)->priv->wrap_height;
 }
 
 
