@@ -1,8 +1,8 @@
 /*
  *      geanywraplabel.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,12 +17,10 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  $Id: geanywraplabel.h 4518 2010-01-01 22:55:18Z eht16 $
  */
 
-#ifndef __GEANY_WRAP_LABEL_H__
-#define __GEANY_WRAP_LABEL_H__
+#ifndef GEANY_WRAP_LABEL_H
+#define GEANY_WRAP_LABEL_H
 
 G_BEGIN_DECLS
 
@@ -43,9 +41,8 @@ typedef struct _GeanyWrapLabelClass  GeanyWrapLabelClass;
 
 GType			geany_wrap_label_get_type			(void);
 GtkWidget*		geany_wrap_label_new				(const gchar *text);
-void			geany_wrap_label_set_text			(GtkLabel *label, const gchar *text);
 
 
 G_END_DECLS
 
-#endif /* __GEANY_WRAP_LABEL_H__ */
+#endif /* GEANY_WRAP_LABEL_H */
