@@ -1,8 +1,8 @@
 /*
  *      keybindings.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,13 +17,13 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: keybindings.h 5641 2011-03-27 16:05:00Z eht16 $
  */
 
 #ifndef GEANY_KEYBINDINGS_H
 #define GEANY_KEYBINDINGS_H 1
 
+G_BEGIN_DECLS
+
 /** Function pointer type used for keybinding callbacks. */
 typedef void (*GeanyKeyCallback) (guint key_id);
 
@@ -42,6 +42,8 @@ typedef struct GeanyKeyBinding
 	GeanyKeyCallback callback;
 	GtkWidget *menu_item;	/**< Optional widget to set an accelerator for, or @c NULL */
 	guint id;
+	guint default_key;
+	GdkModifierType default_mods;
 }
 GeanyKeyBinding;
 
@@ -78,7 +80,7 @@ extern GPtrArray *keybinding_groups;	/* array of GeanyKeyGroup pointers */
  * the existing enum values stay the same. */
 
 /** Keybinding group IDs for use with keybindings_send_command(). */
-/* Groups must be added in this order. */
+/* These IDs are used to lookup a group. */
 enum GeanyKeyGroupID
 {
 	GEANY_KEY_GROUP_FILE,			/**< Group. */
@@ -102,7 +104,7 @@ enum GeanyKeyGroupID
 };
 
 /** Keybinding command IDs for use with keybindings_send_command(). */
-/* These IDs are used to lookup a keybinding; keybindings can be added in any order. */
+/* These IDs are used to lookup a keybinding. */
 enum GeanyKeyBindingID
 {
 	GEANY_KEYS_EDITOR_TRANSPOSELINE,			/**< Keybinding. */
@@ -239,6 +241,10 @@ enum GeanyKeyBindingID
 	GEANY_KEYS_INSERT_LINEAFTER,				/**< Keybinding. */
 	GEANY_KEYS_INSERT_LINEBEFORE,				/**< Keybinding. */
 	GEANY_KEYS_DOCUMENT_REMOVE_MARKERS_INDICATORS,	/**< Keybinding. */
+	GEANY_KEYS_PROJECT_OPEN,					/**< Keybinding. */
+	GEANY_KEYS_PROJECT_NEW,						/**< Keybinding. */
+	GEANY_KEYS_PROJECT_CLOSE,					/**< Keybinding. */
+	GEANY_KEYS_FORMAT_JOINLINES,				/**< Keybinding. */
 	GEANY_KEYS_COUNT	/* must not be used by plugins */
 };
 
@@ -249,6 +255,8 @@ void keybindings_load_keyfile(void);
 
 void keybindings_free(void);
 
+GeanyKeyGroup *keybindings_get_core_group(guint id);
+
 GeanyKeyGroup *keybindings_set_group(GeanyKeyGroup *group, const gchar *section_name,
 		const gchar *label, gsize count, GeanyKeyGroupCallback callback) G_GNUC_WARN_UNUSED_RESULT;
 
@@ -275,4 +283,6 @@ void keybindings_show_shortcuts(void);
 
 gboolean keybindings_check_event(GdkEventKey *ev, GeanyKeyBinding *kb);
 
+G_END_DECLS
+
 #endif
