@@ -1,8 +1,8 @@
 /*
  *      notebook.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: notebook.c 5886 2011-08-06 18:34:14Z colombanw $
  */
 
 /*
@@ -26,6 +24,9 @@
  */
 
 #include "geany.h"
+
+#include <gdk/gdkkeysyms.h>
+
 #include "notebook.h"
 #include "document.h"
 #include "editor.h"
@@ -36,6 +37,7 @@
 #include "callbacks.h"
 #include "utils.h"
 #include "keybindings.h"
+#include "main.h"
 
 #define GEANY_DND_NOTEBOOK_TAB_TYPE	"geany_dnd_notebook_tab"
 
@@ -51,6 +53,14 @@ static GtkTargetEntry files_drop_targets[] = {
 	{ "text/uri-list",	0, 0 }
 };
 
+static const gsize MAX_MRU_DOCS = 20;
+static GQueue *mru_docs = NULL;
+static guint mru_pos = 0;
+
+static gboolean switch_in_progress = FALSE;
+static GtkWidget *switch_dialog = NULL;
+static GtkWidget *switch_dialog_label = NULL;
+
 
 static void
 notebook_page_reordered_cb(GtkNotebook *notebook, GtkWidget *child, guint page_num,
@@ -67,6 +77,228 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data);
 static void setup_tab_dnd(void);
 
 
+static void update_mru_docs_head(GeanyDocument *doc)
+{
+	if (doc)
+	{
+		g_queue_remove(mru_docs, doc);
+		g_queue_push_head(mru_docs, doc);
+
+		if (g_queue_get_length(mru_docs) > MAX_MRU_DOCS)
+			g_queue_pop_tail(mru_docs);
+	}
+}
+
+
+/* before the tab changes, add the current document to the MRU list */
+static void on_notebook_switch_page(GtkNotebook *notebook,
+	GtkNotebookPage *page, guint page_num, gpointer user_data)
+{
+	GeanyDocument *new;
+
+	new = document_get_from_page(page_num);
+
+	/* insert the very first document (when adding the second document
+	 * and switching to it) */
+	if (g_queue_get_length(mru_docs) == 0 && gtk_notebook_get_n_pages(notebook) == 2)
+		update_mru_docs_head(document_get_current());
+
+	if (!switch_in_progress)
+		update_mru_docs_head(new);
+}
+
+
+static void on_document_close(GObject *obj, GeanyDocument *doc)
+{
+	if (! main_status.quitting)
+	{
+		g_queue_remove(mru_docs, doc);
+		/* this prevents the pop up window from showing when there's a single
+		 * document */
+		if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)) == 2)
+			g_queue_clear(mru_docs);
+	}
+}
+
+
+static GtkWidget *ui_minimal_dialog_new(GtkWindow *parent, const gchar *title)
+{
+	GtkWidget *dialog;
+
+	dialog = gtk_window_new(GTK_WINDOW_POPUP);
+
+	if (parent)
+	{
+		gtk_window_set_transient_for(GTK_WINDOW(dialog), parent);
+		gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
+	}
+	gtk_window_set_title(GTK_WINDOW(dialog), title);
+	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+
+	gtk_widget_set_name(dialog, "GeanyDialog");
+	return dialog;
+}
+
+
+static gboolean is_modifier_key(guint keyval)
+{
+	switch (keyval)
+	{
+		case GDK_Shift_L:
+		case GDK_Shift_R:
+		case GDK_Control_L:
+		case GDK_Control_R:
+		case GDK_Meta_L:
+		case GDK_Meta_R:
+		case GDK_Alt_L:
+		case GDK_Alt_R:
+		case GDK_Super_L:
+		case GDK_Super_R:
+		case GDK_Hyper_L:
+		case GDK_Hyper_R:
+			return TRUE;
+		default:
+			return FALSE;
+	}
+}
+
+
+static gboolean on_key_release_event(GtkWidget *widget, GdkEventKey *ev, gpointer user_data)
+{
+	/* user may have rebound keybinding to a different modifier than Ctrl, so check all */
+	if (switch_in_progress && is_modifier_key(ev->keyval))
+	{
+		GeanyDocument *doc;
+
+		switch_in_progress = FALSE;
+
+		if (switch_dialog)
+		{
+			gtk_widget_destroy(switch_dialog);
+			switch_dialog = NULL;
+		}
+
+		doc = document_get_current();
+		update_mru_docs_head(doc);
+		mru_pos = 0;
+		document_check_disk_status(doc, TRUE);
+	}
+	return FALSE;
+}
+
+
+static GtkWidget *create_switch_dialog(void)
+{
+	GtkWidget *dialog, *widget, *vbox;
+
+	dialog = ui_minimal_dialog_new(GTK_WINDOW(main_widgets.window), _("Switch to Document"));
+	gtk_window_set_decorated(GTK_WINDOW(dialog), FALSE);
+	gtk_window_set_default_size(GTK_WINDOW(dialog), 200, -1);
+
+	vbox = gtk_vbox_new(FALSE, 6);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
+	gtk_container_add(GTK_CONTAINER(dialog), vbox);
+
+	widget = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_BUTTON);
+	gtk_container_add(GTK_CONTAINER(vbox), widget);
+
+	widget = gtk_label_new(NULL);
+	gtk_label_set_justify(GTK_LABEL(widget), GTK_JUSTIFY_CENTER);
+	gtk_container_add(GTK_CONTAINER(vbox), widget);
+	switch_dialog_label = widget;
+
+	g_signal_connect(dialog, "key-release-event", G_CALLBACK(on_key_release_event), NULL);
+	return dialog;
+}
+
+
+static void update_filename_label(void)
+{
+	guint i;
+	gchar *msg = NULL;
+	guint queue_length;
+	GeanyDocument *doc;
+
+	if (!switch_dialog)
+	{
+		switch_dialog = create_switch_dialog();
+		gtk_widget_show_all(switch_dialog);
+	}
+
+	queue_length = g_queue_get_length(mru_docs);
+	for (i = mru_pos; (i <= mru_pos + 3) && (doc = g_queue_peek_nth(mru_docs, i % queue_length)); i++)
+	{
+		gchar *basename;
+
+		basename = g_path_get_basename(DOC_FILENAME(doc));
+		if (i == mru_pos)
+			msg = g_markup_printf_escaped ("<b>%s</b>", basename);
+		else if (i % queue_length == mru_pos)    /* && i != mru_pos */
+		{
+			/* We have wrapped around and got to the starting document again */
+			g_free(basename);
+			break;
+		}
+		else
+		{
+			SETPTR(basename, g_markup_printf_escaped ("\n%s", basename));
+			SETPTR(msg, g_strconcat(msg, basename, NULL));
+		}
+		g_free(basename);
+	}
+	gtk_label_set_markup(GTK_LABEL(switch_dialog_label), msg);
+	g_free(msg);
+}
+
+
+static gboolean on_switch_timeout(G_GNUC_UNUSED gpointer data)
+{
+	if (!switch_in_progress || switch_dialog)
+	{
+		return FALSE;
+	}
+
+	update_filename_label();
+	return FALSE;
+}
+
+
+void notebook_switch_tablastused(void)
+{
+	GeanyDocument *last_doc;
+	gboolean switch_start = !switch_in_progress;
+
+	mru_pos += 1;
+	last_doc = g_queue_peek_nth(mru_docs, mru_pos);
+
+	if (! DOC_VALID(last_doc))
+	{
+		utils_beep();
+		mru_pos = 0;
+		last_doc = g_queue_peek_nth(mru_docs, mru_pos);
+	}
+	if (! DOC_VALID(last_doc))
+		return;
+
+	switch_in_progress = TRUE;
+	document_show_tab(last_doc);
+
+	/* if there's a modifier key, we can switch back in MRU order each time unless
+	 * the key is released */
+	if (switch_start)
+		g_timeout_add(600, on_switch_timeout, NULL);
+	else
+		update_filename_label();
+}
+
+
+gboolean notebook_switch_in_progress(void)
+{
+	return switch_in_progress;
+}
+
+
 static gboolean focus_sci(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
 	GeanyDocument *doc = document_get_current();
@@ -177,8 +409,7 @@ static void tab_bar_menu_activate_cb(GtkMenuItem *menuitem, gpointer data)
 	if (! DOC_VALID(doc))
 		return;
 
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-		document_get_notebook_page(doc));
+	document_show_tab(doc);
 }
 
 
@@ -210,11 +441,12 @@ static void on_open_in_new_window_activate(GtkMenuItem *menuitem, gpointer user_
 }
 
 
-static void show_tab_bar_popup_menu(GdkEventButton *event)
+static void show_tab_bar_popup_menu(GdkEventButton *event, GtkWidget *page)
 {
 	GtkWidget *menu_item;
 	static GtkWidget *menu = NULL;
 	GeanyDocument *doc = NULL;
+	gint page_num;
 
 	if (menu == NULL)
 		menu = gtk_menu_new();
@@ -229,24 +461,36 @@ static void show_tab_bar_popup_menu(GdkEventButton *event)
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 
-	doc = document_get_current();
+	if (page != NULL)
+	{
+		page_num = gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook), page);
+		doc = document_get_from_page(page_num);
+	}
+
 	menu_item = ui_image_menu_item_new(GTK_STOCK_OPEN, "Open in New _Window");
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 	g_signal_connect(menu_item, "activate",
 		G_CALLBACK(on_open_in_new_window_activate), doc);
 	/* disable if not on disk */
-	if (!doc->real_path)
+	if (doc == NULL || !doc->real_path)
 		gtk_widget_set_sensitive(menu_item, FALSE);
 
 	menu_item = gtk_separator_menu_item_new();
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 
+	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, NULL);
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(notebook_tab_close_clicked_cb), page);
+	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), (page != NULL));
+
 	menu_item = ui_image_menu_item_new(GTK_STOCK_CLOSE, _("Close Ot_her Documents"));
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
-	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_other_documents1_activate), NULL);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_other_documents1_activate), page);
+	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), (page != NULL));
 
 	menu_item = ui_image_menu_item_new(GTK_STOCK_CLOSE, _("C_lose All"));
 	gtk_widget_show(menu_item);
@@ -273,9 +517,12 @@ static gboolean notebook_tab_bar_click_cb(GtkWidget *widget, GdkEventButton *eve
 			return TRUE;
 		}
 	}
+	/* right-click is also handled here if it happened on the notebook tab bar but not
+	 * on a tab directly */
 	else if (event->button == 3)
 	{
-		show_tab_bar_popup_menu(event);
+		show_tab_bar_popup_menu(event, NULL);
+		return TRUE;
 	}
 	return FALSE;
 }
@@ -300,10 +547,25 @@ void notebook_init()
 	g_signal_connect(main_widgets.notebook, "drag-data-received",
 		G_CALLBACK(on_window_drag_data_received), NULL);
 
+	mru_docs = g_queue_new();
+	g_signal_connect(main_widgets.notebook, "switch-page",
+		G_CALLBACK(on_notebook_switch_page), NULL);
+	g_signal_connect(geany_object, "document-close",
+		G_CALLBACK(on_document_close), NULL);
+
+	/* in case the switch dialog misses an event while drawing the dialog */
+	g_signal_connect(main_widgets.window, "key-release-event", G_CALLBACK(on_key_release_event), NULL);
+
 	setup_tab_dnd();
 }
 
 
+void notebook_free(void)
+{
+	g_queue_free(mru_docs);
+}
+
+
 static void setup_tab_dnd()
 {
 	GtkWidget *notebook = main_widgets.notebook;
@@ -372,6 +634,13 @@ static gboolean notebook_tab_click(GtkWidget *widget, GdkEventButton *event, gpo
 			GEANY_KEYS_NOTEBOOK_SWITCHTABLASTUSED);
 		return TRUE;
 	}
+	/* right-click is first handled here if it happened on a notebook tab */
+	if (event->button == 3)
+	{
+		show_tab_bar_popup_menu(event, data);
+		return TRUE;
+	}
+
 	return FALSE;
 }
 
@@ -473,12 +742,25 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data)
 /* Always use this instead of gtk_notebook_remove_page(). */
 void notebook_remove_page(gint page_num)
 {
-	gint curpage = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
+	gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
 
-	/* Focus the next page, not the previous */
-	if (curpage == page_num && file_prefs.tab_order_ltr)
+	if (page_num == page)
 	{
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), curpage + 1);
+		if (file_prefs.tab_order_ltr)
+			page += 1;
+		else if (page > 0) /* never go negative, it would select the last page */
+			page -= 1;
+
+		if (file_prefs.tab_close_switch_to_mru)
+		{
+			GeanyDocument *last_doc;
+
+			last_doc = g_queue_peek_nth(mru_docs, 0);
+			if (DOC_VALID(last_doc))
+				page = document_get_notebook_page(last_doc);
+		}
+
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), page);
 	}
 
 	/* now remove the page (so we don't temporarily switch to the previous page) */
@@ -508,5 +790,3 @@ on_window_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
 	}
 	gtk_drag_finish(drag_context, success, FALSE, event_time);
 }
-
-
