@@ -1,8 +1,8 @@
 /*
  *      notebook.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: notebook.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 #ifndef GEANY_NOTEBOOK_H
@@ -26,10 +24,20 @@
 
 void notebook_init(void);
 
+void notebook_free(void);
+
 /* Returns page number of notebook page, or -1 on error */
 gint notebook_new_tab(GeanyDocument *doc);
 
 /* Always use this instead of gtk_notebook_remove_page(). */
 void notebook_remove_page(gint page_num);
 
+/* Switch notebook to the last used tab. Can be called repeatedly to get to the
+ * previous tabs. */
+void notebook_switch_tablastused(void);
+
+/* Returns TRUE when MRU tab switch is in progress (i.e. not at the final 
+ * document yet). */
+gboolean notebook_switch_in_progress(void);
+
 #endif
