@@ -1,8 +1,8 @@
 /*
  *      plugindata.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: plugindata.h 5859 2011-06-18 14:54:50Z colombanw $
  */
 
 /**
@@ -32,8 +30,10 @@
  * when making changes (see 'Keeping the plugin ABI stable' in the HACKING file). */
 
 
-#ifndef PLUGINDATA_H
-#define PLUGINDATA_H
+#ifndef GEANY_PLUGINDATA_H
+#define GEANY_PLUGINDATA_H
+
+G_BEGIN_DECLS
 
 /* Compatibility for sharing macros between API and core.
  * First include geany.h, then plugindata.h, then other API headers. */
@@ -41,6 +41,7 @@
 #define GEANY(symbol_name) geany->symbol_name
 
 #include "editor.h"	/* GeanyIndentType */
+#include "build.h"  /* GeanyBuildGroup, GeanyBuildSource, GeanyBuildCmdEntries enums */
 
 
 /** The Application Programming Interface (API) version, incremented
@@ -54,14 +55,14 @@
  * @warning You should not test for values below 200 as previously
  * @c GEANY_API_VERSION was defined as an enum value, not a macro.
  */
-#define GEANY_API_VERSION 211
+#define GEANY_API_VERSION 215
 
 /** The Application Binary Interface (ABI) version, incremented whenever
  * existing fields in the plugin data types have to be changed or reordered.
  * Changing this forces all plugins to be recompiled before Geany can load them. */
 /* This should usually stay the same if fields are only appended, assuming only pointers to
  * structs and not structs themselves are declared by plugins. */
-#define GEANY_ABI_VERSION 68
+#define GEANY_ABI_VERSION 69
 
 
 /** Defines a function to check the plugin is safe to load.
@@ -268,14 +269,15 @@ typedef struct GeanyFunctions
 	struct SearchFuncs			*p_search;			/**< See search.h */
 	struct HighlightingFuncs	*p_highlighting;	/**< See highlighting.h */
 	struct FiletypeFuncs		*p_filetypes;		/**< See filetypes.h */
-	struct NavQueueFuncs        *p_navqueue;		/**< See navqueue.h */
-	struct EditorFuncs        	*p_editor;			/**< See editor.h */
-	struct MainFuncs        	*p_main;			/**< See main.h */
-	struct PluginFuncs        	*p_plugin;			/**< See pluginutils.c */
+	struct NavQueueFuncs		*p_navqueue;		/**< See navqueue.h */
+	struct EditorFuncs			*p_editor;			/**< See editor.h */
+	struct MainFuncs			*p_main;			/**< See main.h */
+	struct PluginFuncs			*p_plugin;			/**< See pluginutils.c */
 	struct ScintillaFuncs		*p_scintilla;		/**< See ScintillaFuncs */
 	struct MsgWinFuncs			*p_msgwin;			/**< See msgwindow.h */
 	struct StashFuncs			*p_stash;			/**< See stash.h */
 	struct SymbolsFuncs			*p_symbols;			/**< See symbols.h */
+	struct BuildFuncs			*p_build;			/**< See build.h */
 }
 GeanyFunctions;
 
@@ -479,6 +481,7 @@ typedef struct UIUtilsFuncs
 				const gchar *text, gint history_len);
 	void		(*ui_menu_add_document_items_sorted) (GtkMenu *menu, struct GeanyDocument *active,
 				GCallback callback, GCompareFunc compare_func);
+	const gchar* (*ui_lookup_stock_label)(const gchar *stock_id);
 }
 UIUtilsFuncs;
 
@@ -523,8 +526,8 @@ MsgWinFuncs;
 /* See encodings.h */
 typedef struct EncodingFuncs
 {
-	gchar*			(*encodings_convert_to_utf8) (const gchar *buffer, gsize size, gchar **used_encoding);
-	gchar* 			(*encodings_convert_to_utf8_from_charset) (const gchar *buffer, gsize size,
+	gchar*			(*encodings_convert_to_utf8) (const gchar *buffer, gssize size, gchar **used_encoding);
+	gchar* 			(*encodings_convert_to_utf8_from_charset) (const gchar *buffer, gssize size,
 													 const gchar *charset, gboolean fast);
 	const gchar*	(*encodings_get_charset_from_index) (gint idx);
 }
@@ -698,6 +701,7 @@ typedef struct StashFuncs
 			const gchar *property_name, GType type);
 	void (*stash_group_display)(struct StashGroup *group, GtkWidget *owner);
 	void (*stash_group_update)(struct StashGroup *group, GtkWidget *owner);
+	void (*stash_group_free_settings)(struct StashGroup *group);
 }
 StashFuncs;
 
@@ -709,6 +713,19 @@ typedef struct SymbolsFuncs
 }
 SymbolsFuncs;
 
+/* See build.h */
+typedef struct BuildFuncs
+{
+	void (*build_activate_menu_item)(const GeanyBuildGroup grp, const guint cmd);
+	const gchar *(*build_get_current_menu_item)(const GeanyBuildGroup grp, const guint cmd,
+			const GeanyBuildCmdEntries field);
+	void (*build_remove_menu_item)(const GeanyBuildSource src, const GeanyBuildGroup grp,
+			const gint cmd);
+	void (*build_set_menu_item)(const GeanyBuildSource src, const GeanyBuildGroup grp,
+			const guint cmd, const GeanyBuildCmdEntries field, const gchar *value);
+	guint (*build_get_group_count)(const GeanyBuildGroup grp);
+}
+BuildFuncs;
 
 /* Deprecated aliases */
 #ifndef GEANY_DISABLE_DEPRECATED
@@ -725,4 +742,6 @@ SymbolsFuncs;
 
 #endif	/* GEANY_DISABLE_DEPRECATED */
 
+G_END_DECLS
+
 #endif
