@@ -1,8 +1,8 @@
 /*
  *      pluginprivate.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,13 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: pluginprivate.h 5650 2011-03-29 23:20:14Z colombanw $
  */
 
 
-#ifndef PLUGINPRIVATE_H
-#define PLUGINPRIVATE_H
+#ifndef GEANY_PLUGINPRIVATE_H
+#define GEANY_PLUGINPRIVATE_H
 
 #include "plugindata.h"
 #include "ui_utils.h"	/* GeanyAutoSeparator */
@@ -64,4 +62,4 @@ GeanyPluginPrivate;
 typedef GeanyPluginPrivate Plugin;	/* shorter alias */
 
 
-#endif /* PLUGINPRIVATE_H */
+#endif /* GEANY_PLUGINPRIVATE_H */
