@@ -1,8 +1,8 @@
 /*
  *      plugins.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: plugins.c 5886 2011-08-06 18:34:14Z colombanw $
  */
 
 /* Code to manage, load and unload plugins. */
@@ -48,7 +46,7 @@
 #include "msgwindow.h"
 #include "prefs.h"
 #include "geanywraplabel.h"
-#include "build.h"
+/* #include "build.h" included in plugindata.h so it can use enums */
 #include "encodings.h"
 #include "search.h"
 #include "highlighting.h"
@@ -77,6 +75,7 @@ static GList *failed_plugins_list = NULL;	/* plugins the user wants active but c
 
 static GtkWidget *menu_separator = NULL;
 
+static gchar *get_plugin_path(void);
 static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data);
 
 
@@ -248,7 +247,8 @@ static UIUtilsFuncs uiutils_funcs = {
 	&ui_is_keyval_enter_or_return,
 	&ui_get_gtk_settings_integer,
 	&ui_combo_box_add_to_history,
-	&ui_menu_add_document_items_sorted
+	&ui_menu_add_document_items_sorted,
+	&ui_lookup_stock_label
 };
 
 static DialogFuncs dialog_funcs = {
@@ -346,13 +346,22 @@ static StashFuncs stash_funcs = {
 	&stash_group_add_entry,
 	&stash_group_add_widget_property,
 	&stash_group_display,
-	&stash_group_update
+	&stash_group_update,
+	&stash_group_free_settings
 };
 
 static SymbolsFuncs symbols_funcs = {
 	&symbols_get_context_separator
 };
 
+static BuildFuncs build_funcs = {
+	&build_activate_menu_item,
+	&build_get_current_menu_item,
+	&build_remove_menu_item,
+	&build_set_menu_item,
+	&build_get_group_count
+};
+
 static GeanyFunctions geany_functions = {
 	&doc_funcs,
 	&sci_funcs,
@@ -375,7 +384,8 @@ static GeanyFunctions geany_functions = {
 	&scintilla_funcs,
 	&msgwin_funcs,
 	&stash_funcs,
-	&symbols_funcs
+	&symbols_funcs,
+	&build_funcs
 };
 
 static GeanyData geany_data;
@@ -660,7 +670,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 		geany_debug("Plugin \"%s\" already loaded.", fname);
 		if (add_to_list)
 		{
-			/* do not add the to list twice */
+			/* do not add to the list twice */
 			if (g_list_find(plugin_list, plugin) != NULL)
 				return NULL;
 
@@ -841,6 +851,60 @@ plugin_free(Plugin *plugin)
 }
 
 
+static gchar *get_custom_plugin_path(const gchar *plugin_path_config,
+									 const gchar *plugin_path_system)
+{
+	gchar *plugin_path_custom;
+
+	if (!NZV(prefs.custom_plugin_path))
+		return NULL;
+
+	plugin_path_custom = utils_get_locale_from_utf8(prefs.custom_plugin_path);
+	utils_tidy_path(plugin_path_custom);
+
+	/* check whether the custom plugin path is one of the system or user plugin paths
+	 * and abort if so */
+	if (utils_str_equal(plugin_path_custom, plugin_path_config) ||
+		utils_str_equal(plugin_path_custom, plugin_path_system))
+	{
+		g_free(plugin_path_custom);
+		return NULL;
+	}
+	return plugin_path_custom;
+}
+
+
+/* all 3 paths Geany looks for plugins in can change (even system path on Windows)
+ * so we need to check active plugins are in the right place before loading */
+static gboolean check_plugin_path(const gchar *fname)
+{
+	gchar *plugin_path_config;
+	gchar *plugin_path_system;
+	gchar *plugin_path_custom;
+	gboolean ret = FALSE;
+
+	plugin_path_config = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", NULL);
+	if (g_str_has_prefix(fname, plugin_path_config))
+		ret = TRUE;
+
+	plugin_path_system = get_plugin_path();
+	if (g_str_has_prefix(fname, plugin_path_system))
+		ret = TRUE;
+
+	plugin_path_custom = get_custom_plugin_path(plugin_path_config, plugin_path_system);
+	if (plugin_path_custom)
+	{
+		if (g_str_has_prefix(fname, plugin_path_custom))
+			ret = TRUE;
+
+		g_free(plugin_path_custom);
+	}
+	g_free(plugin_path_config);
+	g_free(plugin_path_system);
+	return ret;
+}
+
+
 /* load active plugins at startup */
 static void
 load_active_plugins(void)
@@ -856,7 +920,7 @@ load_active_plugins(void)
 
 		if (NZV(fname) && g_file_test(fname, G_FILE_TEST_EXISTS))
 		{
-			if (plugin_new(fname, TRUE, FALSE) == NULL)
+			if (!check_plugin_path(fname) || plugin_new(fname, TRUE, FALSE) == NULL)
 				failed_plugins_list = g_list_prepend(failed_plugins_list, g_strdup(fname));
 		}
 	}
@@ -888,7 +952,7 @@ load_plugins_from_path(const gchar *path)
 	g_slist_free(list);
 
 	if (count)
-		geany_debug("Found %d plugin(s) in '%s'.", count, path);
+		geany_debug("Added %d plugin(s) in '%s'.", count, path);
 }
 
 
@@ -908,25 +972,12 @@ static gchar *get_plugin_path(void)
 }
 
 
-static gboolean validate_custom_plugin_path(const gchar *plugin_path_custom,
-											const gchar *plugin_path_config,
-											const gchar *plugin_path_system)
-{
-	/* check whether the custom plugin path is one of the system or user plugin paths
-	 * and abort if so */
-	if (utils_str_equal(plugin_path_custom, plugin_path_config) ||
-		utils_str_equal(plugin_path_custom, plugin_path_system))
-		return FALSE;
-
-	return TRUE;
-}
-
-
 /* Load (but don't initialize) all plugins for the Plugin Manager dialog */
 static void load_all_plugins(void)
 {
 	gchar *plugin_path_config;
 	gchar *plugin_path_system;
+	gchar *plugin_path_custom;
 
 	plugin_path_config = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", NULL);
 	plugin_path_system = get_plugin_path();
@@ -935,14 +986,10 @@ static void load_all_plugins(void)
 	load_plugins_from_path(plugin_path_config);
 
 	/* load plugins from a custom path */
-	if (NZV(prefs.custom_plugin_path))
+	plugin_path_custom = get_custom_plugin_path(plugin_path_config, plugin_path_system);
+	if (plugin_path_custom)
 	{
-		gchar *plugin_path_custom = utils_get_locale_from_utf8(prefs.custom_plugin_path);
-		utils_tidy_path(plugin_path_custom);
-
-		if (validate_custom_plugin_path(plugin_path_custom, plugin_path_config, plugin_path_system))
-			load_plugins_from_path(plugin_path_custom);
-
+		load_plugins_from_path(plugin_path_custom);
 		g_free(plugin_path_custom);
 	}
 
@@ -1050,6 +1097,11 @@ static void on_save_settings(GKeyFile *config)
 void plugins_init(void)
 {
 	StashGroup *group;
+	gchar *path;
+
+	path = get_plugin_path();
+	geany_debug("System plugin path: %s", path);
+	g_free(path);
 
 	group = stash_group_new("plugins");
 	configuration_add_pref_group(group, TRUE);
@@ -1106,7 +1158,7 @@ enum
 {
 	PLUGIN_COLUMN_CHECK = 0,
 	PLUGIN_COLUMN_NAME,
-	PLUGIN_COLUMN_FILE,
+	PLUGIN_COLUMN_DESCRIPTION,
 	PLUGIN_COLUMN_PLUGIN,
 	PLUGIN_N_COLUMNS,
 	PM_BUTTON_CONFIGURE,
@@ -1119,7 +1171,7 @@ typedef struct
 	GtkWidget *tree;
 	GtkListStore *store;
 	GtkWidget *plugin_label;
-	GtkWidget *description_label;
+	GtkWidget *filename_label;
 	GtkWidget *author_label;
 	GtkWidget *configure_button;
 	GtkWidget *help_button;
@@ -1159,7 +1211,7 @@ static void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data
 			/* Translators: <plugin name> <plugin version> */
 			text = g_strdup_printf(_("%s %s"), pi->name, pi->version);
 			gtk_label_set_text(GTK_LABEL(pm_widgets.plugin_label), text);
-			gtk_label_set_text(GTK_LABEL(pm_widgets.description_label), pi->description);
+			gtk_label_set_text(GTK_LABEL(pm_widgets.filename_label), p->filename);
 			gtk_label_set_text(GTK_LABEL(pm_widgets.author_label), pi->author);
 			g_free(text);
 
@@ -1245,7 +1297,7 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 	text_renderer = gtk_cell_renderer_text_new();
 	g_object_set(text_renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	column = gtk_tree_view_column_new_with_attributes(
-		_("File"), text_renderer, "text", PLUGIN_COLUMN_FILE, NULL);
+		_("Description"), text_renderer, "text", PLUGIN_COLUMN_DESCRIPTION, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
@@ -1264,7 +1316,7 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter, PLUGIN_COLUMN_CHECK, FALSE,
 				PLUGIN_COLUMN_NAME, _("No plugins available."),
-				PLUGIN_COLUMN_FILE, "", PLUGIN_COLUMN_PLUGIN, NULL, -1);
+				PLUGIN_COLUMN_DESCRIPTION, "", PLUGIN_COLUMN_PLUGIN, NULL, -1);
 	}
 	else
 	{
@@ -1277,7 +1329,7 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 			gtk_list_store_set(store, &iter,
 				PLUGIN_COLUMN_CHECK, is_active_plugin(p),
 				PLUGIN_COLUMN_NAME, p->info.name,
-				PLUGIN_COLUMN_FILE, p->filename,
+				PLUGIN_COLUMN_DESCRIPTION, p->info.description,
 				PLUGIN_COLUMN_PLUGIN, p,
 				-1);
 		}
@@ -1398,14 +1450,14 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 	table = gtk_table_new(3, 2, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 6);
 	pm_widgets.plugin_label = geany_wrap_label_new(NULL);
-	pm_widgets.description_label = geany_wrap_label_new(NULL);
+	pm_widgets.filename_label = geany_wrap_label_new(NULL);
 	pm_widgets.author_label = geany_wrap_label_new(NULL);
 	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Plugin:")), 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Description:")), 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Author(s):")), 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Author(s):")), 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Filename:")), 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach(GTK_TABLE(table), pm_widgets.plugin_label, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-	gtk_table_attach(GTK_TABLE(table), pm_widgets.description_label, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-	gtk_table_attach(GTK_TABLE(table), pm_widgets.author_label, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), pm_widgets.author_label, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), pm_widgets.filename_label, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
 	desc_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(desc_win),
