@@ -1,8 +1,8 @@
 /*
  *      pluginutils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,13 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: pluginutils.h 5650 2011-03-29 23:20:14Z colombanw $
  */
 
 
-#ifndef PLUGINUTILS_H
-#define PLUGINUTILS_H
+#ifndef GEANY_PLUGINUTILS_H
+#define GEANY_PLUGINUTILS_H
 
 #ifdef HAVE_PLUGINS
 
@@ -57,4 +55,4 @@ struct GeanyKeyGroup *plugin_set_key_group(struct GeanyPlugin *plugin,
 void plugin_show_configure(struct GeanyPlugin *plugin);
 
 #endif /* HAVE_PLUGINS */
-#endif /* PLUGINUTILS_H */
+#endif /* GEANY_PLUGINUTILS_H */
