@@ -1,8 +1,8 @@
 /*
  *      prefs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: prefs.c 5943 2011-09-21 16:03:11Z ntrel $
  */
 
 /*
@@ -48,7 +46,6 @@
 #include "msgwindow.h"
 #include "keyfile.h"
 #include "keybindings.h"
-#include "interface.h"
 #include "encodings.h"
 #include "project.h"
 #include "editor.h"
@@ -83,10 +80,11 @@ static gboolean edited = FALSE;
 
 static GtkTreeView *various_treeview = NULL;
 
+static GeanyKeyBinding *kb_index(guint gidx, guint kid);
 static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path, gchar *new_text, gpointer user_data);
 static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data);
 static void kb_grab_key_dialog_response_cb(GtkWidget *dialog, gint response, gpointer user_data);
-static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
+static gboolean kb_find_duplicate(GtkWidget *parent, GtkTreeIter *old_iter,
 		guint key, GdkModifierType mods, const gchar *shortcut);
 static void on_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data);
@@ -129,10 +127,10 @@ static void prefs_action(PrefCallbackAction action)
 	switch (action)
 	{
 		case PREF_DISPLAY:
-			stash_tree_display(pref_groups, various_treeview);
+			stash_tree_display(various_treeview);
 			break;
 		case PREF_UPDATE:
-			stash_tree_update(pref_groups, various_treeview);
+			stash_tree_update(various_treeview);
 			break;
 	}
 }
@@ -143,7 +141,9 @@ enum
 	KB_TREE_ACTION,
 	KB_TREE_SHORTCUT,
 	KB_TREE_INDEX,
-	KB_TREE_EDITABLE
+	KB_TREE_EDITABLE,
+	KB_TREE_WEIGHT,
+	KB_TREE_COLUMNS
 };
 
 
@@ -216,7 +216,8 @@ static void kb_show_popup_menu(GtkWidget *widget, GdkEventButton *event)
 {
 	GtkWidget *item;
 	static GtkWidget *menu = NULL;
-	gint button, event_time;
+	guint button;
+	guint32 event_time;
 
 	if (menu == NULL)
 	{
@@ -281,17 +282,20 @@ static void kb_init_tree(void)
 
 	tree = GTK_TREE_VIEW(ui_lookup_widget(ui_widgets.prefs_dialog, "treeview7"));
 
-	store = gtk_tree_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
+	store = gtk_tree_store_new(KB_TREE_COLUMNS,
+		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
 	renderer = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes(_("Action"), renderer, "text", KB_TREE_ACTION, NULL);
+	column = gtk_tree_view_column_new_with_attributes(_("Action"), renderer,
+		"text", KB_TREE_ACTION, "weight", KB_TREE_WEIGHT, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Shortcut"), renderer,
-		"text", KB_TREE_SHORTCUT, "editable", KB_TREE_EDITABLE, NULL);
+		"text", KB_TREE_SHORTCUT, "editable", KB_TREE_EDITABLE,
+		"weight", KB_TREE_WEIGHT, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	/* set policy settings for the scrolled window around the treeview again, because glade
@@ -308,11 +312,33 @@ static void kb_init_tree(void)
 }
 
 
+static void kb_set_shortcut(GtkTreeStore *store, GtkTreeIter *iter,
+		guint key, GdkModifierType mods)
+{
+	gchar *key_string = gtk_accelerator_name(key, mods);
+	GtkTreeIter parent;
+	guint kid, gid;
+	GeanyKeyBinding *kb;
+	gboolean bold;
+
+	gtk_tree_store_set(store, iter, KB_TREE_SHORTCUT, key_string, -1);
+	g_free(key_string);
+
+	gtk_tree_model_get(GTK_TREE_MODEL(store), iter, KB_TREE_INDEX, &kid, -1);
+	gtk_tree_model_iter_parent(GTK_TREE_MODEL(store), &parent, iter);
+	gtk_tree_model_get(GTK_TREE_MODEL(store), &parent, KB_TREE_INDEX, &gid, -1);
+	kb = kb_index(gid, kid);
+	bold = key != kb->default_key || mods != kb->default_mods;
+	gtk_tree_store_set(store, iter, KB_TREE_WEIGHT,
+		bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL, -1);
+}
+
+
 static void kb_init(void)
 {
 	GtkTreeIter parent, iter;
 	gsize g, i;
-	gchar *key_string, *label;
+	gchar *label;
 	GeanyKeyGroup *group;
 	GeanyKeyBinding *kb;
 
@@ -328,11 +354,10 @@ static void kb_init(void)
 		foreach_ptr_array(kb, i, group->key_items)
 		{
 			label = keybindings_get_label(kb);
-			key_string = gtk_accelerator_name(kb->key, kb->mods);
 			gtk_tree_store_append(store, &iter, &parent);
 			gtk_tree_store_set(store, &iter, KB_TREE_ACTION, label,
-				KB_TREE_SHORTCUT, key_string, KB_TREE_EDITABLE, TRUE, KB_TREE_INDEX, kb->id, -1);
-			g_free(key_string);
+				KB_TREE_EDITABLE, TRUE, KB_TREE_INDEX, kb->id, -1);
+			kb_set_shortcut(store, &iter, kb->key, kb->mods);
 			g_free(label);
 		}
 	}
@@ -753,6 +778,16 @@ static void prefs_init_dialog(void)
 }
 
 
+/* note: uses group index, not group id, unlike keybindings_lookup_item(). */
+static GeanyKeyBinding *kb_index(guint gidx, guint kid)
+{
+	GeanyKeyGroup *group;
+
+	group = g_ptr_array_index(keybinding_groups, gidx);
+	return keybindings_get_item(group, kid);
+}
+
+
 /* read the treeview shortcut fields into keybindings */
 static void kb_update(void)
 {
@@ -783,7 +818,7 @@ static void kb_update(void)
 			gtk_tree_model_get(model, &child, KB_TREE_INDEX, &kid, KB_TREE_SHORTCUT, &str, -1);
 			gtk_accelerator_parse(str, &key, &mods);
 			g_free(str);
-			kb = keybindings_lookup_item(gid, kid);
+			kb = kb_index(gid, kid);
 			if (kb->key != key || kb->mods != mods)
 				keybindings_update_combo(kb, key, mods);
 
@@ -808,7 +843,7 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	{
 		GtkWidget *widget;
 		guint i;
-		guint autoclose_brackets[5];
+		gboolean autoclose_brackets[5];
 		gboolean old_invert_all = interface_prefs.highlighting_invert_all;
 		gboolean old_sidebar_pos = interface_prefs.sidebar_pos;
 		GeanyDocument *doc = document_get_current();
@@ -983,7 +1018,7 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 
 		/* Editor settings */
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_toggle_mark");
-		setptr(editor_prefs.comment_toggle_mark,
+		SETPTR(editor_prefs.comment_toggle_mark,
 			gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_long_line");
@@ -1071,11 +1106,11 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		autoclose_brackets[4] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		editor_prefs.autoclose_chars =
-		  (autoclose_brackets[0] ? GEANY_AC_PARENTHESIS : 0)
-		| (autoclose_brackets[1] ? GEANY_AC_CBRACKET : 0)
-		| (autoclose_brackets[2] ? GEANY_AC_SBRACKET : 0)
-		| (autoclose_brackets[3] ? GEANY_AC_SQUOTE : 0)
-		| (autoclose_brackets[4] ? GEANY_AC_DQUOTE : 0);
+		  (autoclose_brackets[0] ? GEANY_AC_PARENTHESIS : 0u)
+		| (autoclose_brackets[1] ? GEANY_AC_CBRACKET : 0u)
+		| (autoclose_brackets[2] ? GEANY_AC_SBRACKET : 0u)
+		| (autoclose_brackets[3] ? GEANY_AC_SQUOTE : 0u)
+		| (autoclose_brackets[4] ? GEANY_AC_DQUOTE : 0u);
 
 		/* Tools Settings */
 
@@ -1229,7 +1264,6 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		ui_update_view_editor_menu_items();
 
 		/* various preferences */
-		doc = document_get_current();
 		ui_save_buttons_toggle((doc != NULL) ? doc->changed : FALSE);
 		msgwin_show_hide_tabs();
 		ui_update_statusbar(doc, -1);
@@ -1255,7 +1289,7 @@ static void on_color_button_choose_cb(GtkColorButton *widget, gpointer user_data
 	GdkColor color;
 
 	gtk_color_button_get_color(widget, &color);
-	setptr(editor_prefs.long_line_color, utils_get_hex_from_color(&color));
+	SETPTR(editor_prefs.long_line_color, utils_get_hex_from_color(&color));
 }
 
 
@@ -1271,7 +1305,7 @@ static void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 			if (strcmp(fontbtn, interface_prefs.tagbar_font) == 0)
 				break;
 
-			setptr(interface_prefs.tagbar_font, g_strdup(fontbtn));
+			SETPTR(interface_prefs.tagbar_font, g_strdup(fontbtn));
 			for (i = 0; i < documents_array->len; i++)
 			{
 				GeanyDocument *doc = documents[i];
@@ -1289,7 +1323,7 @@ static void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 		{
 			if (strcmp(fontbtn, interface_prefs.msgwin_font) == 0)
 				break;
-			setptr(interface_prefs.msgwin_font, g_strdup(fontbtn));
+			SETPTR(interface_prefs.msgwin_font, g_strdup(fontbtn));
 			ui_widget_modify_font_from_string(msgwindow.tree_compiler, interface_prefs.msgwin_font);
 			ui_widget_modify_font_from_string(msgwindow.tree_msg, interface_prefs.msgwin_font);
 			ui_widget_modify_font_from_string(msgwindow.tree_status, interface_prefs.msgwin_font);
@@ -1305,38 +1339,19 @@ static void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 }
 
 
-static GeanyKeyBinding *kb_lookup_kb_from_iter(G_GNUC_UNUSED GtkTreeModel *model, GtkTreeIter *iter)
-{
-	guint group_idx, keybinding_idx;
-	GtkTreeIter parent;
-
-	/* get kb index */
-	gtk_tree_model_get(GTK_TREE_MODEL(store), iter, KB_TREE_INDEX, &keybinding_idx, -1);
-
-	/* lookup the parent to get group index */
-	gtk_tree_model_iter_parent(GTK_TREE_MODEL(store), &parent, iter);
-	gtk_tree_model_get(GTK_TREE_MODEL(store), &parent, KB_TREE_INDEX, &group_idx, -1);
-
-	return keybindings_lookup_item(group_idx, keybinding_idx);
-}
-
-
 static void kb_change_iter_shortcut(GtkTreeIter *iter, const gchar *new_text)
 {
 	guint lkey;
 	GdkModifierType lmods;
-	GeanyKeyBinding *kb;
 
 	gtk_accelerator_parse(new_text, &lkey, &lmods);
 
-	kb = kb_lookup_kb_from_iter(GTK_TREE_MODEL(store), iter);
-
-	if (kb_find_duplicate(ui_widgets.prefs_dialog, kb, lkey, lmods, new_text))
+	if (kb_find_duplicate(ui_widgets.prefs_dialog, iter, lkey, lmods, new_text))
 		return;
 
 	/* set the values here, because of the above check, setting it in
 	 * gtk_accelerator_parse would return a wrong key combination if it is duplicate */
-	gtk_tree_store_set(store, iter, KB_TREE_SHORTCUT, new_text, -1);
+	kb_set_shortcut(store, iter, lkey, lmods);
 
 	edited = TRUE;
 }
@@ -1361,7 +1376,7 @@ static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext,
 static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data)
 {
 	gchar *str;
-	gint state;
+	guint state;
 
 	state = event->state & gtk_accelerator_get_default_mod_mask();
 
@@ -1389,70 +1404,55 @@ static void kb_grab_key_dialog_response_cb(GtkWidget *dialog, gint response, G_G
 }
 
 
-/* Look for a (1st-level) child of parent whose KB_TREE_INDEX matches i,
- * setting iter to point to the node if found.
- * If parent is NULL, look for a parent node whose KB_TREE_INDEX matches i. */
-static gboolean kb_find_child_iter(GtkTreeIter *parent, guint i, GtkTreeIter *iter)
-{
-	GtkTreeModel *model = GTK_TREE_MODEL(store);
-	guint idx;
-
-	/* get first child of parent */
-	if (! gtk_tree_model_iter_children(model, iter, parent))
-		return FALSE;
-
-	while (TRUE)	/* foreach child */
-	{
-		gtk_tree_model_get(model, iter, KB_TREE_INDEX, &idx, -1);
-		if (idx == i)
-			return TRUE;
-		if (! gtk_tree_model_iter_next(model, iter))
-			return FALSE;	/* no more children */
-	}
-}
-
-
-static void kb_clear_tree_shortcut(gsize group_id, gsize keybinding_id)
-{
-	GtkTreeIter parent;
-	GtkTreeIter child;
-
-	/* find parent kb group */
-	if (! kb_find_child_iter(NULL, group_id, &parent))
-		return;
-
-	/* find child kb node*/
-	if (! kb_find_child_iter(&parent, keybinding_id, &child))
-		return;
-
-	gtk_tree_store_set(store, &child, KB_TREE_SHORTCUT, NULL, -1);	/* clear shortcut */
-}
-
-
 /* test if the entered key combination is already used
  * returns true if cancelling duplicate */
-static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
+static gboolean kb_find_duplicate(GtkWidget *parent, GtkTreeIter *old_iter,
 		guint key, GdkModifierType mods, const gchar *shortcut)
 {
-	gsize g, i;
-	GeanyKeyGroup *group;
-	GeanyKeyBinding *kb;
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	GtkTreeIter parent_iter;
+	gchar *kb_str;
+	guint kb_key;
+	GdkModifierType kb_mods;
 
 	/* allow duplicate if there is no key combination */
 	if (key == 0 && mods == 0)
 		return FALSE;
 
-	foreach_ptr_array(group, g, keybinding_groups)
+	/* don't check if the new keybinding is the same as the old one */
+	gtk_tree_model_get(model, old_iter, KB_TREE_SHORTCUT, &kb_str, -1);
+	if (kb_str)
 	{
-		foreach_ptr_array(kb, i, group->key_items)
+		gtk_accelerator_parse(kb_str, &kb_key, &kb_mods);
+		g_free(kb_str);
+		if (key == kb_key && mods == kb_mods)
+			return FALSE;
+	}
+
+	if (! gtk_tree_model_get_iter_first(model, &parent_iter))
+		return FALSE;
+	do	/* foreach top level */
+	{
+		GtkTreeIter iter;
+
+		if (! gtk_tree_model_iter_children(model, &iter, &parent_iter))
+			continue;
+		do	/* foreach children */
 		{
-			/* search another item with the same key,
-			 * but don't search the key we're looking for(!) */
-			if (kb->key == key && kb->mods == mods
-				&& ! (kb->key == search_kb->key && kb->mods == search_kb->mods))
+			gtk_tree_model_get(model, &iter, KB_TREE_SHORTCUT, &kb_str, -1);
+			if (! kb_str)
+				continue;
+
+			gtk_accelerator_parse(kb_str, &kb_key, &kb_mods);
+			g_free(kb_str);
+			/* search another item with the same key and modifiers */
+			if (kb_key == key && kb_mods == mods)
 			{
-				gchar *label = keybindings_get_label(kb);
-				gint ret = dialogs_show_prompt(parent,
+				gchar *label;
+				gint ret;
+
+				gtk_tree_model_get(model, &iter, KB_TREE_ACTION, &label, -1);
+				ret = dialogs_show_prompt(parent,
 					_("_Allow"), GTK_RESPONSE_APPLY,
 					GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
 					_("_Override"), GTK_RESPONSE_YES,
@@ -1464,14 +1464,17 @@ static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
 
 				if (ret == GTK_RESPONSE_YES)
 				{
-					kb_clear_tree_shortcut(g, kb->id);
+					kb_set_shortcut(store, &iter, 0, 0);	/* clear shortcut */
 					/* carry on looking for other duplicates if overriding */
 					continue;
 				}
 				return ret == GTK_RESPONSE_NO;
 			}
 		}
+		while (gtk_tree_model_iter_next(model, &iter));
 	}
+	while (gtk_tree_model_iter_next(model, &parent_iter));
+
 	return FALSE;
 }
 
@@ -1612,11 +1615,20 @@ static gboolean prefs_dialog_key_press_response_cb(GtkWidget *dialog, GdkEventKe
 }
 
 
+static void list_store_append_text(GtkListStore *list, const gchar *text)
+{
+	GtkTreeIter iter;
+
+	gtk_list_store_append(list, &iter);
+	gtk_list_store_set(list, &iter, 0, text, -1);
+}
+
+
 void prefs_show_dialog(void)
 {
 	if (ui_widgets.prefs_dialog == NULL)
 	{
-		GtkWidget *combo_new, *combo_open, *combo_eol;
+		GtkListStore *encoding_list, *eol_list;
 		GtkWidget *label;
 		guint i;
 		gchar *encoding_string;
@@ -1629,24 +1641,20 @@ void prefs_show_dialog(void)
 		gtk_window_set_icon(GTK_WINDOW(ui_widgets.prefs_dialog), pb);
 		g_object_unref(pb);	/* free our reference */
 
-		/* init the default file encoding combo box */
-		combo_new = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_new_encoding");
-		combo_open = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_open_encoding");
-		gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(combo_new), 3);
-		gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(combo_open), 3);
+		/* init the file encoding combo boxes */
+		encoding_list = ui_builder_get_object("encoding_list");
 		for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
 		{
 			encoding_string = encodings_to_string(&encodings[i]);
-			gtk_combo_box_append_text(GTK_COMBO_BOX(combo_new), encoding_string);
-			gtk_combo_box_append_text(GTK_COMBO_BOX(combo_open), encoding_string);
+			list_store_append_text(encoding_list, encoding_string);
 			g_free(encoding_string);
 		}
 
 		/* init the eol character combo box */
-		combo_eol = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_eol");
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_eol), utils_get_eol_name(SC_EOL_CRLF));
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_eol), utils_get_eol_name(SC_EOL_CR));
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_eol), utils_get_eol_name(SC_EOL_LF));
+		eol_list = ui_builder_get_object("eol_list");
+		list_store_append_text(eol_list, utils_get_eol_name(SC_EOL_CRLF));
+		list_store_append_text(eol_list, utils_get_eol_name(SC_EOL_CR));
+		list_store_append_text(eol_list, utils_get_eol_name(SC_EOL_LF));
 
 		/* add manually GeanyWrapLabels because they can't be added with Glade */
 		/* page Tools */
