@@ -1,8 +1,8 @@
 /*
  *      printing.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: printing.c 5884 2011-08-06 18:33:04Z colombanw $
  */
 
 
@@ -53,7 +51,7 @@ PrintingPrefs printing_prefs;
 #define ADD_ATTR(l, a) \
 	pango_attr_list_insert((l), (a)); \
 	(a)->start_index = 0; \
-	(a)->end_index = -1;
+	(a)->end_index = G_MAXUINT;
 
 
 enum
@@ -919,4 +917,3 @@ void printing_print_doc(GeanyDocument *doc)
 	else
 		print_external(doc);
 }
-
