@@ -1,8 +1,8 @@
 /*
  *      project.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: project.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 
 #ifndef GEANY_PROJECT_H
 #define GEANY_PROJECT_H 1
 
+G_BEGIN_DECLS
+
 #define GEANY_PROJECT_EXT				"geany"
 
 
@@ -86,6 +86,8 @@ gboolean project_load_file_with_session(const gchar *locale_file_name);
 gchar *project_get_base_path(void);
 
 
+const struct GeanyFilePrefs *project_get_file_prefs(void);
+
 void project_save_prefs(GKeyFile *config);
 
 void project_load_prefs(GKeyFile *config);
@@ -94,4 +96,6 @@ void project_setup_prefs(void);
 
 void project_apply_prefs(void);
 
+G_END_DECLS
+
 #endif
