@@ -1,8 +1,8 @@
 /*
  *      projectprivate.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
- *
- * $Id: projectprivate.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 
@@ -30,6 +28,10 @@
 typedef struct GeanyProjectPrivate
 {
 	struct GeanyIndentPrefs *indentation;
+	gboolean	final_new_line;
+	gboolean	strip_trailing_spaces;
+	gboolean	replace_tabs;
+	gboolean	ensure_convert_new_lines;
 }
 GeanyProjectPrivate;
 
