@@ -1,8 +1,8 @@
 /*
  *      sidebar.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: sidebar.c 5856 2011-06-17 22:52:43Z colombanw $
  */
 
 /*
@@ -59,8 +57,10 @@ static struct
 	GtkWidget *reload;
 	GtkWidget *show_paths;
 	GtkWidget *find_in_files;
+	GtkWidget *expand_all;
+	GtkWidget *collapse_all;
 }
-doc_items = {NULL, NULL, NULL, NULL, NULL};
+doc_items = {NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 
 enum
 {
@@ -216,7 +216,7 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 		if (doc->priv->tag_tree == NULL)
 		{
 			doc->priv->tag_store = gtk_tree_store_new(
-				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, TM_TYPE_TAG, G_TYPE_STRING, G_TYPE_BOOLEAN);
+				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, TM_TYPE_TAG, G_TYPE_STRING);
 			doc->priv->tag_tree = gtk_tree_view_new();
 			prepare_taglist(doc->priv->tag_tree, doc->priv->tag_store);
 			gtk_widget_show(doc->priv->tag_tree);
@@ -336,13 +336,23 @@ static gboolean find_tree_iter_dir(GtkTreeIter *iter, const gchar *dir)
 
 	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, DOCUMENTS_SHORTNAME, &name, -1);
 
-	result = utils_str_equal(name, dir);
+	result = utils_filenamecmp(name, dir) == 0;
 	g_free(name);
 
 	return result;
 }
 
 
+static gboolean utils_filename_has_prefix(const gchar *str, const gchar *prefix)
+{
+	gchar *head = g_strndup(str, strlen(prefix));
+	gboolean ret = utils_filenamecmp(head, prefix) == 0;
+
+	g_free(head);
+	return ret;
+}
+
+
 static gchar *get_doc_folder(const gchar *path)
 {
 	gchar *tmp_dirname = g_strdup(path);
@@ -362,7 +372,7 @@ static gchar *get_doc_folder(const gchar *path)
 			project_base_path[len-1] = '\0';
 
 		/* check whether the dir name matches or uses the project base path */
-		if (g_str_has_prefix(tmp_dirname, project_base_path))
+		if (utils_filename_has_prefix(tmp_dirname, project_base_path))
 		{
 			rest = tmp_dirname + len;
 			if (*rest == G_DIR_SEPARATOR || *rest == '\0')
@@ -377,7 +387,7 @@ static gchar *get_doc_folder(const gchar *path)
 		dirname = tmp_dirname;
 
 		/* If matches home dir, replace with tilde */
-		if (home_dir && *home_dir != 0 && g_str_has_prefix(dirname, home_dir))
+		if (NZV(home_dir) && utils_filename_has_prefix(dirname, home_dir))
 		{
 			rest = dirname + strlen(home_dir);
 			if (*rest == G_DIR_SEPARATOR || *rest == '\0')
@@ -520,18 +530,12 @@ void sidebar_openfiles_update(GeanyDocument *doc)
 
 void sidebar_openfiles_update_all()
 {
-	guint i, page_count;
-	GeanyDocument *doc;
+	guint i;
 
 	gtk_tree_store_clear(store_openfiles);
-	page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
-	for (i = 0; i < page_count; i++)
+	foreach_document (i)
 	{
-		doc = document_get_from_page(i);
-		if (G_UNLIKELY(doc == NULL))
-			continue;
-
-		sidebar_openfiles_add(doc);
+		sidebar_openfiles_add(documents[i]);
 	}
 }
 
@@ -542,12 +546,9 @@ void sidebar_remove_document(GeanyDocument *doc)
 
 	if (GTK_IS_WIDGET(doc->priv->tag_tree))
 	{
-		gtk_widget_destroy(doc->priv->tag_tree);
-		if (GTK_IS_TREE_VIEW(doc->priv->tag_tree))
-		{
-			/* Because it was ref'd in sidebar_update_tag_list, it needs unref'ing */
-			g_object_unref((gpointer)doc->priv->tag_tree);
-		}
+		gtk_widget_destroy(doc->priv->tag_tree); /* make GTK release its references, if any */
+		/* Because it was ref'd in sidebar_update_tag_list, it needs unref'ing */
+		g_object_unref(doc->priv->tag_tree);
 		doc->priv->tag_tree = NULL;
 	}
 }
@@ -602,7 +603,7 @@ void sidebar_add_common_menu_items(GtkMenu *menu)
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("H_ide Sidebar"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-		gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_MENU));
+		gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_hide_sidebar), NULL);
@@ -657,13 +658,24 @@ static void on_find_in_files(GtkMenuItem *menuitem, gpointer user_data)
 }
 
 
+static void on_openfiles_expand_collapse(GtkMenuItem *menuitem, gpointer user_data)
+{
+	gboolean expand = GPOINTER_TO_INT(user_data);
+
+	if (expand)
+		gtk_tree_view_expand_all(GTK_TREE_VIEW(tv.tree_openfiles));
+	else
+		gtk_tree_view_collapse_all(GTK_TREE_VIEW(tv.tree_openfiles));
+}
+
+
 static void create_openfiles_popup_menu(void)
 {
 	GtkWidget *item;
 
 	openfiles_popup_menu = gtk_menu_new();
 
-	item = gtk_image_menu_item_new_from_stock("gtk-close", NULL);
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 	g_signal_connect(item, "activate",
@@ -674,7 +686,7 @@ static void create_openfiles_popup_menu(void)
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 
-	item = gtk_image_menu_item_new_from_stock("gtk-save", NULL);
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE, NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 	g_signal_connect(item, "activate",
@@ -683,7 +695,7 @@ static void create_openfiles_popup_menu(void)
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Reload"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-		gtk_image_new_from_stock("gtk-revert-to-saved", GTK_ICON_SIZE_MENU));
+		gtk_image_new_from_stock(GTK_STOCK_REVERT_TO_SAVED, GTK_ICON_SIZE_MENU));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 	g_signal_connect(item, "activate",
@@ -711,6 +723,22 @@ static void create_openfiles_popup_menu(void)
 	g_signal_connect(doc_items.show_paths, "activate",
 			G_CALLBACK(on_openfiles_show_paths_activate), NULL);
 
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
+
+	doc_items.expand_all = ui_image_menu_item_new(GTK_STOCK_ADD, _("_Expand All"));
+	gtk_widget_show(doc_items.expand_all);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), doc_items.expand_all);
+	g_signal_connect(doc_items.expand_all, "activate",
+					 G_CALLBACK(on_openfiles_expand_collapse), GINT_TO_POINTER(TRUE));
+
+	doc_items.collapse_all = ui_image_menu_item_new(GTK_STOCK_REMOVE, _("_Collapse All"));
+	gtk_widget_show(doc_items.collapse_all);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), doc_items.collapse_all);
+	g_signal_connect(doc_items.collapse_all, "activate",
+					 G_CALLBACK(on_openfiles_expand_collapse), GINT_TO_POINTER(FALSE));
+
 	sidebar_add_common_menu_items(GTK_MENU(openfiles_popup_menu));
 }
 
@@ -840,9 +868,7 @@ static gboolean openfiles_go_to_selection(GtkTreeSelection *selection, guint key
 			return FALSE;	/* parent */
 
 		/* switch to the doc and grab the focus */
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-			gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
-			(GtkWidget*) doc->editor->sci));
+		document_show_tab(doc);
 		if (keyval != GDK_space)
 			change_focus_to_editor(doc, tv.tree_openfiles);
 	}
@@ -1003,8 +1029,9 @@ static void documents_menu_update(GtkTreeSelection *selection)
 	gtk_widget_set_sensitive(doc_items.find_in_files, sel);
 	g_free(shortname);
 
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(doc_items.show_paths),
-		documents_show_paths);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(doc_items.show_paths), documents_show_paths);
+	gtk_widget_set_sensitive(doc_items.expand_all, documents_show_paths);
+	gtk_widget_set_sensitive(doc_items.collapse_all, documents_show_paths);
 }
 
 
@@ -1060,11 +1087,8 @@ void sidebar_finalize(void)
 {
 	if (WIDGET(tv.default_tag_tree))
 	{
-		g_object_unref(tv.default_tag_tree);
-		/* This is not exactly clean, default_tag_tree's ref_count is 2 when it is shown,
-		 * 1 oherwise. We should probably handle the ref_count more accurate. */
-		if (WIDGET(tv.default_tag_tree))
-			gtk_widget_destroy(tv.default_tag_tree);
+		gtk_widget_destroy(tv.default_tag_tree); /* make GTK release its references, if any... */
+		g_object_unref(tv.default_tag_tree); /* ...and release our own */
 	}
 	if (WIDGET(tv.popup_taglist))
 		gtk_widget_destroy(tv.popup_taglist);
