@@ -1,8 +1,8 @@
 /*
  *      socket.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      Copyright 2006 Hiroyuki Yamamoto (author of Sylpheed)
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: socket.c 5975 2011-09-30 11:14:23Z ntrel $
  */
 
 /*
@@ -42,7 +40,7 @@
  * The command window is only available on Windows and takes no additional data, instead it
  * writes back a Windows handle (HWND) for the main window to set it to the foreground (focus).
  *
- * At the moment the commands window, doclist, open, line and column are available.
+ * At the moment the commands window, doclist, open, openro, line and column are available.
  *
  * About the socket files on Unix-like systems:
  * Geany creates a socket in /tmp (or any other directory returned by g_get_tmp_dir()) and
@@ -88,7 +86,7 @@
 #include "utils.h"
 #include "dialogs.h"
 #include "encodings.h"
-
+#include "project.h"
 
 
 #ifdef G_OS_WIN32
@@ -148,7 +146,10 @@ static void send_open_command(gint sock, gint argc, gchar **argv)
 		g_free(col);
 	}
 
-	socket_fd_write_all(sock, "open\n", 5);
+	if (cl_options.readonly) /* append "ro" to denote readonly status for new docs */
+		socket_fd_write_all(sock, "openro\n", 7);
+	else
+		socket_fd_write_all(sock, "open\n", 5);
 
 	for (i = 1; i < argc && argv[i] != NULL; i++)
 	{
@@ -280,8 +281,12 @@ gint socket_init(gint argc, gchar **argv)
 	/* these lines are taken from dcopc.c in kdelibs */
 	if ((p = strrchr(display_name, '.')) > strrchr(display_name, ':') && p != NULL)
 		*p = '\0';
-	while ((p = strchr(display_name, ':')) != NULL)
-		*p = '_';
+	/* remove characters that may not be acceptable in a filename */
+	for (p = display_name; *p; p++)
+	{
+		if (*p == ':' || *p == '/')
+			*p = '_';
+	}
 
 	if (socket_info.file_name == NULL)
 		socket_info.file_name = g_strdup_printf("%s%cgeany_socket_%s_%s",
@@ -414,7 +419,7 @@ static gint socket_fd_open_unix(const gchar *path)
 	{	/* if real_path is not writable for us, fall back to ~/.config/geany/geany_socket_*_* */
 		/* instead of creating a symlink and print a warning */
 		g_warning("Socket %s could not be written, using %s as fallback.", real_path, path);
-		setptr(real_path, g_strdup(path));
+		SETPTR(real_path, g_strdup(path));
 	}
 	/* create a symlink in e.g. ~/.config/geany/geany_socket_hostname__0 to /tmp/geany_socket.499602d2 */
 	else if (symlink(real_path, path) != 0)
@@ -550,7 +555,7 @@ static void handle_input_filename(const gchar *buf)
 
 	/* we never know how the input is encoded, so do the best auto detection we can */
 	if (! g_utf8_validate(buf, -1, NULL))
-		utf8_filename = encodings_convert_to_utf8(buf, (gsize) -1, NULL);
+		utf8_filename = encodings_convert_to_utf8(buf, -1, NULL);
 	else
 		utf8_filename = g_strdup(buf);
 
@@ -558,7 +563,10 @@ static void handle_input_filename(const gchar *buf)
 	if (locale_filename)
 	{
 		if (g_str_has_suffix(locale_filename, ".geany"))
-			main_load_project_from_command_line(locale_filename, TRUE);
+		{
+			if (project_ask_close())
+				main_load_project_from_command_line(locale_filename, TRUE);
+		}
 		else
 			main_handle_filename(locale_filename);
 	}
@@ -588,7 +596,7 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 	gint fd, sock;
 	gchar buf[BUFFER_LENGTH];
 	struct sockaddr_in caddr;
-	guint caddr_len = sizeof(caddr);
+	socklen_t caddr_len = sizeof(caddr);
 	GtkWidget *window = data;
 	gboolean popup = FALSE;
 
@@ -600,6 +608,7 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 	{
 		if (strncmp(buf, "open", 4) == 0)
 		{
+			cl_options.readonly = strncmp(buf+4, "ro", 2) == 0; /* open in readonly? */
 			while (socket_fd_gets(sock, buf, sizeof(buf)) != -1 && *buf != '.')
 			{
 				handle_input_filename(g_strstrip(buf));
@@ -638,7 +647,8 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 #ifdef G_OS_WIN32
 		else if (strncmp(buf, "window", 6) == 0)
 		{
-			HWND hwnd = (HWND) gdk_win32_drawable_get_handle(GDK_DRAWABLE(window->window));
+			HWND hwnd = (HWND) gdk_win32_drawable_get_handle(
+				GDK_DRAWABLE(gtk_widget_get_window(window)));
 			socket_fd_write(sock, (gchar *)&hwnd, sizeof(hwnd));
 		}
 #endif
@@ -651,11 +661,12 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 		 * gtk_window_present() really bring the main window into the foreground on some
 		 * window managers like Gnome's metacity.
 		 * Code taken from Gedit. */
-		gdk_x11_window_set_user_time(window->window, gdk_x11_get_server_time(window->window));
+		gdk_x11_window_set_user_time(gtk_widget_get_window(window),
+			gdk_x11_get_server_time(gtk_widget_get_window(window)));
 #endif
 		gtk_window_present(GTK_WINDOW(window));
 #ifdef G_OS_WIN32
-		gdk_window_show(window->window);
+		gdk_window_show(gtk_widget_get_window(window));
 #endif
 	}
 
@@ -790,5 +801,3 @@ gint socket_fd_write(gint fd, const gchar *buf, gint len)
 
 
 #endif
-
-
