@@ -1,8 +1,8 @@
 /*
  *      socket.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: socket.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 
@@ -32,7 +30,7 @@ struct socket_info_struct
 	gchar		*file_name;
 	GIOChannel	*read_ioc;
 	gint 		 lock_socket;
-	gint 		 lock_socket_tag;
+	guint 		 lock_socket_tag;
 };
 
 extern struct socket_info_struct socket_info;
