@@ -1,8 +1,8 @@
 /*
  *	  stash.h - this file is part of Geany, a fast and lightweight IDE
  *
- *	  Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *	  Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *	  Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *	  Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *	  This program is free software; you can redistribute it and/or modify
  *	  it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  *	  along with this program; if not, write to the Free Software
  *	  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *	  MA 02110-1301, USA.
- *
- * $Id: stash.h 5942 2011-09-21 15:50:36Z ntrel $
  */
 
 #ifndef GEANY_STASH_H
 #define GEANY_STASH_H
 
+G_BEGIN_DECLS
+
 /** Opaque type for a group of settings. */
 typedef struct StashGroup StashGroup;
 
@@ -60,6 +60,8 @@ gboolean stash_group_load_from_file(StashGroup *group, const gchar *filename);
 gint stash_group_save_to_file(StashGroup *group, const gchar *filename,
 		GKeyFileFlags flags);
 
+void stash_group_free_settings(StashGroup *group);
+
 void stash_group_free(StashGroup *group);
 
 
@@ -94,8 +96,10 @@ void stash_group_update(StashGroup *group, GtkWidget *owner);
 
 void stash_tree_setup(GPtrArray *group_array, GtkTreeView *tree);
 
-void stash_tree_display(GPtrArray *group_array, GtkTreeView *tree);
+void stash_tree_display(GtkTreeView *tree);
 
-void stash_tree_update(GPtrArray *group_array, GtkTreeView *tree);
+void stash_tree_update(GtkTreeView *tree);
+
+G_END_DECLS
 
 #endif
