@@ -1,8 +1,9 @@
 /*
  *      symbols.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2011-2012 Colomban Wendling <ban(at)herbesfolles(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +18,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: symbols.c 5949 2011-09-22 18:04:03Z colombanw $
  */
 
 /**
@@ -55,6 +54,7 @@
 #include "ui_utils.h"
 #include "editor.h"
 #include "sciwrappers.h"
+#include "filetypesprivate.h"
 
 
 const guint TM_GLOBAL_TYPE_MASK =
@@ -119,6 +119,9 @@ static void load_c_ignore_tags(void)
 
 	if (g_file_get_contents(path, &content, NULL, NULL))
 	{
+		/* historically we ignore the glib _DECLS for tag generation */
+		SETPTR(content, g_strconcat("G_BEGIN_DECLS G_END_DECLS\n", content, NULL));
+
 		g_strfreev(c_tags_ignore);
 		c_tags_ignore = g_strsplit_set(content, " \n\r", -1);
 		g_free(content);
@@ -162,7 +165,7 @@ static gboolean symbols_load_global_tags(const gchar *tags_file, GeanyFiletype *
 
 /* Ensure that the global tags file(s) for the file_type_idx filetype is loaded.
  * This provides autocompletion, calltips, etc. */
-void symbols_global_tags_loaded(gint file_type_idx)
+void symbols_global_tags_loaded(guint file_type_idx)
 {
 	TagFileInfo *tfi;
 	gint tag_type;
@@ -373,7 +376,7 @@ symbols_find_tm_tag(const GPtrArray *tags, const gchar *tag_name)
 
 
 static TMTag *find_work_object_tag(const TMWorkObject *workobj,
-		const gchar *tag_name, gint type)
+		const gchar *tag_name, guint type)
 {
 	GPtrArray *tags;
 	TMTag *tmtag;
@@ -395,7 +398,7 @@ static TMTag *find_work_object_tag(const TMWorkObject *workobj,
 }
 
 
-static TMTag *find_workspace_tag(const gchar *tag_name, gint type)
+static TMTag *find_workspace_tag(const gchar *tag_name, guint type)
 {
 	guint j;
 	const GPtrArray *work_objects = NULL;
@@ -514,6 +517,7 @@ struct TreeviewSymbols
 	GtkTreeIter		 tag_variable;
 	GtkTreeIter		 tag_namespace;
 	GtkTreeIter		 tag_struct;
+	GtkTreeIter		 tag_interface;
 	GtkTreeIter		 tag_type;
 	GtkTreeIter		 tag_other;
 } tv_iters;
@@ -530,6 +534,7 @@ static void init_tag_iters(void)
 	tv_iters.tag_variable.stamp = -1;
 	tv_iters.tag_namespace.stamp = -1;
 	tv_iters.tag_struct.stamp = -1;
+	tv_iters.tag_interface.stamp = -1;
 	tv_iters.tag_type.stamp = -1;
 	tv_iters.tag_other.stamp = -1;
 }
@@ -560,14 +565,15 @@ static GdkPixbuf *get_tag_icon(const gchar *icon_name)
 
 /* finds the next iter at any level
  * @param iter in/out, the current iter, will be changed to the next one
+ * @param down whether to try the child iter
  * @return TRUE if there @p iter was set, or FALSE if there is no next iter */
-static gboolean next_iter(GtkTreeModel *model, GtkTreeIter *iter)
+static gboolean next_iter(GtkTreeModel *model, GtkTreeIter *iter, gboolean down)
 {
 	GtkTreeIter guess;
 	GtkTreeIter copy = *iter;
 
 	/* go down if the item has children */
-	if (gtk_tree_model_iter_children(model, &guess, iter))
+	if (down && gtk_tree_model_iter_children(model, &guess, iter))
 		*iter = guess;
 	/* or to the next item at the same level */
 	else if (gtk_tree_model_iter_next(model, &copy))
@@ -656,7 +662,7 @@ tag_list_add_groups(GtkTreeStore *tree_store, ...)
 			gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_ICON, icon, -1);
 			g_object_unref(icon);
 		}
-		gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_NAME, title, SYMBOLS_COLUMN_VALID, TRUE, -1);
+		gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_NAME, title, -1);
 	}
 	va_end(args);
 }
@@ -770,7 +776,7 @@ static void add_top_level_items(GeanyDocument *doc)
 		case GEANY_FILETYPES_PHP:
 		{
 			tag_list_add_groups(tag_store,
-				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
+				&(tv_iters.tag_interface), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
 				&(tv_iters.tag_macro), _("Constants"), "classviewer-macro",
@@ -869,7 +875,7 @@ static void add_top_level_items(GeanyDocument *doc)
 		{
 			tag_list_add_groups(tag_store,
 				&(tv_iters.tag_namespace), _("Package"), "classviewer-namespace",
-				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
+				&(tv_iters.tag_interface), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
 				&(tv_iters.tag_member), _("Members"), "classviewer-member",
@@ -881,7 +887,7 @@ static void add_top_level_items(GeanyDocument *doc)
 		{
 			tag_list_add_groups(tag_store,
 				&(tv_iters.tag_namespace), _("Package"), "classviewer-namespace",
-				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
+				&(tv_iters.tag_interface), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
 				&(tv_iters.tag_member), _("Properties"), "classviewer-member",
@@ -894,7 +900,7 @@ static void add_top_level_items(GeanyDocument *doc)
 		case GEANY_FILETYPES_HAXE:
 		{
 			tag_list_add_groups(tag_store,
-				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
+				&(tv_iters.tag_interface), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
 				&(tv_iters.tag_type), _("Types"), "classviewer-macro",
@@ -920,7 +926,7 @@ static void add_top_level_items(GeanyDocument *doc)
 		{
 			tag_list_add_groups(tag_store,
 				&(tv_iters.tag_namespace), _("Module"), "classviewer-class",
-				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
+				&(tv_iters.tag_interface), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
 				&(tv_iters.tag_member), _("Subroutines"), "classviewer-method",
 				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
@@ -971,6 +977,7 @@ static void add_top_level_items(GeanyDocument *doc)
 
 			tag_list_add_groups(tag_store,
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
+				&(tv_iters.tag_interface), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
 				&(tv_iters.tag_member), _("Members"), "classviewer-member",
 				&(tv_iters.tag_struct), _("Structs"), "classviewer-struct",
@@ -1024,7 +1031,7 @@ static const gchar *get_symbol_name(GeanyDocument *doc, const TMTag *tag, gboole
 
 	if (! doc_is_utf8)
 		utf8_name = encodings_convert_to_utf8_from_charset(tag->name,
-			(gsize) -1, doc->encoding, TRUE);
+			-1, doc->encoding, TRUE);
 	else
 		utf8_name = tag->name;
 
@@ -1066,12 +1073,12 @@ static gchar *get_symbol_tooltip(GeanyDocument *doc, const TMTag *tag)
 		! utils_str_equal(doc->encoding, "UTF-8") &&
 		! utils_str_equal(doc->encoding, "None"))
 	{
-		setptr(utf8_name,
-			encodings_convert_to_utf8_from_charset(utf8_name, (gsize) -1, doc->encoding, TRUE));
+		SETPTR(utf8_name,
+			encodings_convert_to_utf8_from_charset(utf8_name, -1, doc->encoding, TRUE));
 	}
 
 	if (utf8_name != NULL)
-		setptr(utf8_name, g_markup_escape_text(utf8_name, -1));
+		SETPTR(utf8_name, g_markup_escape_text(utf8_name, -1));
 
 	return utf8_name;
 }
@@ -1142,11 +1149,13 @@ static GtkTreeIter *get_tag_type_iter(TMTagType tag_type, filetype_id ft_id)
 		}
 		case tm_tag_union_t:
 		case tm_tag_struct_t:
-		case tm_tag_interface_t:
 		{
 			iter = &tv_iters.tag_struct;
 			break;
 		}
+		case tm_tag_interface_t:
+			iter = &tv_iters.tag_interface;
+			break;
 		case tm_tag_variable_t:
 		{
 			iter = &tv_iters.tag_variable;
@@ -1223,165 +1232,317 @@ static guint tag_hash(gconstpointer v)
 }
 
 
-static GHashTable *build_iter_table(GtkTreeStore *store)
+/* like gtk_tree_view_expand_to_path() but with an iter */
+static void tree_view_expand_to_iter(GtkTreeView *view, GtkTreeIter *iter)
 {
+	GtkTreeModel *model = gtk_tree_view_get_model(view);
+	GtkTreePath *path = gtk_tree_model_get_path(model, iter);
+
+	gtk_tree_view_expand_to_path(view, path);
+	gtk_tree_path_free(path);
+}
+
+
+/* like gtk_tree_store_remove() but finds the next iter at any level */
+static gboolean tree_store_remove_row(GtkTreeStore *store, GtkTreeIter *iter)
+{
+	GtkTreeIter parent;
+	gboolean has_parent;
+	gboolean cont;
+
+	has_parent = gtk_tree_model_iter_parent(GTK_TREE_MODEL(store), &parent, iter);
+	cont = gtk_tree_store_remove(store, iter);
+	/* if there is no next at this level but there is a parent iter, continue from it */
+	if (! cont && has_parent)
+	{
+		*iter = parent;
+		cont = next_iter(GTK_TREE_MODEL(store), iter, FALSE);
+	}
+
+	return cont;
+}
+
+
+/* adds a new element in the parent table if it's key is known.
+ * duplicates are kept */
+static void update_parents_table(GHashTable *table, const TMTag *tag, const gchar *parent_name,
+		const GtkTreeIter *iter)
+{
+	GList **list;
+	if (g_hash_table_lookup_extended(table, tag->name, NULL, (gpointer *) &list) &&
+		! utils_str_equal(parent_name, tag->name) /* prevent Foo::Foo from making parent = child */)
+	{
+		if (! list)
+		{
+			list = g_slice_alloc(sizeof *list);
+			*list = NULL;
+			g_hash_table_insert(table, tag->name, list);
+		}
+		*list = g_list_prepend(*list, g_slice_dup(GtkTreeIter, iter));
+	}
+}
+
+
+static void free_iter_slice_list(gpointer data)
+{
+	GList **list = data;
+
+	if (list)
+	{
+		GList *node;
+		foreach_list(node, *list)
+			g_slice_free(GtkTreeIter, node->data);
+		g_list_free(*list);
+		g_slice_free1(sizeof *list, list);
+	}
+}
+
+
+/* inserts a @data in @table on key @tag.
+ * previous data is not overwritten if the key is duplicated, but rather the
+ * two values are kept in a list
+ *
+ * table is: GHashTable<TMTag, GList<GList<TMTag>>> */
+static void tags_table_insert(GHashTable *table, TMTag *tag, GList *data)
+{
+	GList *list = g_hash_table_lookup(table, tag);
+	list = g_list_prepend(list, data);
+	g_hash_table_insert(table, tag, list);
+}
+
+
+/* looks up the entry in @table that better matches @tag.
+ * if there are more than one candidate, the one that has closest line position to @tag is chosen */
+static GList *tags_table_lookup(GHashTable *table, TMTag *tag)
+{
+	GList *data = NULL;
+	GList *node = g_hash_table_lookup(table, tag);
+	if (node)
+	{
+		glong delta;
+		data = node->data;
+
+#define TAG_DELTA(a, b) ABS((glong) TM_TAG(a)->atts.entry.line - (glong) TM_TAG(b)->atts.entry.line)
+
+		delta = TAG_DELTA(((GList *) node->data)->data, tag);
+		for (node = node->next; node; node = node->next)
+		{
+			glong d = TAG_DELTA(((GList *) node->data)->data, tag);
+
+			if (d < delta)
+			{
+				data = node->data;
+				delta = d;
+			}
+		}
+
+#undef TAG_DELTA
+
+	}
+	return data;
+}
+
+
+/* removes the element at @tag from @table.
+ * @tag must be the exact pointer used at insertion time */
+static void tags_table_remove(GHashTable *table, TMTag *tag)
+{
+	GList *list = g_hash_table_lookup(table, tag);
+	if (list)
+	{
+		GList *node;
+		foreach_list(node, list)
+		{
+			if (((GList *) node->data)->data == tag)
+				break;
+		}
+		list = g_list_delete_link(list, node);
+		if (list)
+			g_hash_table_insert(table, tag, list);
+		else
+			g_hash_table_remove(table, tag);
+	}
+}
+
+
+/*
+ * Updates the tag tree for a document with the tags in *list.
+ * @param doc a document
+ * @param tags a pointer to a GList* holding the tags to add/update.  This
+ *             list may be updated, removing updated elements.
+ *
+ * The update is done in two passes:
+ * 1) walking the current tree, update tags that still exist and remove the
+ *    obsolescent ones;
+ * 2) walking the remaining (non updated) tags, adds them in the list.
+ *
+ * For better performances, we use 2 hash tables:
+ * - one containing all the tags for lookup in the first pass (actually stores a
+ *   reference in the tags list for removing it efficiently), avoiding list search
+ *   on each tag;
+ * - the other holding "tag-name":row references for tags having children, used to
+ *   lookup for a parent in both passes, avoiding tree traversal.
+ */
+static void update_tree_tags(GeanyDocument *doc, GList **tags)
+{
+	GtkTreeStore *store = doc->priv->tag_store;
 	GtkTreeModel *model = GTK_TREE_MODEL(store);
-	GHashTable *table;
+	GHashTable *parents_table;
+	GHashTable *tags_table;
 	GtkTreeIter iter;
+	gboolean cont;
+	GList *item;
 
-	table = g_hash_table_new_full(tag_hash, tag_equal,
-			(GDestroyNotify)tm_tag_unref, (GDestroyNotify)gtk_tree_path_free);
+	/* Build hash tables holding tags and parents */
+	/* parent table holds "tag-name":GtkTreeIter */
+	parents_table = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, free_iter_slice_list);
+	/* tags table is another representation of the @tags list, TMTag:GList<TMTag> */
+	tags_table = g_hash_table_new_full(tag_hash, tag_equal, NULL, NULL);
+	foreach_list(item, *tags)
+	{
+		TMTag *tag = item->data;
+		const gchar *name;
 
-	if (!gtk_tree_model_get_iter_first(model, &iter))
-		return table;
-	do
+		tags_table_insert(tags_table, tag, item);
+
+		name = get_parent_name(tag, doc->file_type->id);
+		if (name)
+			g_hash_table_insert(parents_table, (gpointer) name, NULL);
+	}
+
+	/* First pass, update existing rows or delete them.
+	 * It is OK to delete them since we walk top down so we would remove
+	 * parents before checking for their children, thus never implicitly
+	 * deleting an updated child */
+	cont = gtk_tree_model_get_iter_first(model, &iter);
+	while (cont)
 	{
 		TMTag *tag;
 
 		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_TAG, &tag, -1);
-		if (tag)
-			g_hash_table_insert(table, tag, gtk_tree_model_get_path(model, &iter));
-	}
-	while (next_iter(model, &iter));
-
-	return table;
-}
-
-
-static gboolean find_iter(GtkTreeStore *store, GHashTable *iter_hash, const TMTag *tag,
-		GtkTreeIter *iter)
-{
-	GtkTreePath *path;
-
-	path = g_hash_table_lookup(iter_hash, tag);
-	if (path)
-	{
-		GtkTreeIter tmp;
-		gboolean valid;
-
-		if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &tmp, path))
-			return FALSE;
-		gtk_tree_model_get(GTK_TREE_MODEL(store), &tmp, SYMBOLS_COLUMN_VALID, &valid, -1);
-		/* if the row is valid it has already been updated, so it's simply a duplicate */
-		if (!valid)
+		if (! tag) /* most probably a toplevel, skip it */
+			cont = next_iter(model, &iter, TRUE);
+		else
 		{
-			*iter = tmp;
-			return TRUE;
+			GList *found_item;
+
+			found_item = tags_table_lookup(tags_table, tag);
+			if (! found_item) /* tag doesn't exist, remove it */
+				cont = tree_store_remove_row(store, &iter);
+			else /* tag still exist, update it */
+			{
+				const gchar *name;
+				const gchar *parent_name;
+				TMTag *found = found_item->data;
+
+				parent_name = get_parent_name(found, doc->file_type->id);
+				/* if parent is unknown, ignore it */
+				if (parent_name && ! g_hash_table_lookup(parents_table, parent_name))
+					parent_name = NULL;
+
+				/* only update fields that (can) have changed (name that holds line
+				 * number, and the tag itself) */
+				name = get_symbol_name(doc, found, parent_name != NULL);
+				gtk_tree_store_set(store, &iter,
+						SYMBOLS_COLUMN_NAME, name,
+						SYMBOLS_COLUMN_TAG, found,
+						-1);
+
+				update_parents_table(parents_table, found, parent_name, &iter);
+
+				/* remove the updated tag from the table and list */
+				tags_table_remove(tags_table, found);
+				*tags = g_list_delete_link(*tags, found_item);
+
+				cont = next_iter(model, &iter, TRUE);
+			}
+
+			tm_tag_unref(tag);
 		}
 	}
 
-	return FALSE;
-}
-
-
-static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *parent_hash,
-		GHashTable *iter_hash)
-{
-	filetype_id ft_id = doc->file_type->id;
-	GtkTreeStore *tree_store = doc->priv->tag_store;
-	GtkTreeIter *parent = NULL;
-
-	parent = get_tag_type_iter(tag->type, ft_id);
-
-	if (G_LIKELY(parent))
+	/* Second pass, now we have a tree cleaned up from invalid rows,
+	 * we simply add new ones */
+	foreach_list (item, *tags)
 	{
-		const gchar *name;
-		const gchar *parent_name = get_parent_name(tag, ft_id);
-		GtkTreeIter iter;
-		GtkTreeIter *child = NULL;
- 		GdkPixbuf *icon = NULL;
-		gchar *tooltip;
+		TMTag *tag = item->data;
+		GtkTreeIter *parent;
 
- 		child = &iter;
-		icon = get_child_icon(tree_store, parent);
-
-		if (parent_name)
+		parent = get_tag_type_iter(tag->type, doc->file_type->id);
+		if (G_UNLIKELY(! parent))
+			geany_debug("Missing symbol-tree parent iter for type %d!", tag->type);
+		else
 		{
-			GtkTreeIter *parent_search =
-				(GtkTreeIter *)g_hash_table_lookup(parent_hash, parent_name);
+			gboolean expand;
+			const gchar *name;
+			const gchar *parent_name;
+			gchar *tooltip;
+			GdkPixbuf *icon = get_child_icon(store, parent);
 
-			if (parent_search)
-				parent = parent_search;
-			else
-				parent_name = NULL;
-		}
+			parent_name = get_parent_name(tag, doc->file_type->id);
+			if (parent_name)
+			{
+				GList **candidates;
+				GtkTreeIter *parent_search = NULL;
 
-		/* check if the current tag is a parent of other tags */
-		if (g_hash_table_lookup_extended(parent_hash, tag->name, NULL, NULL) &&
-			!utils_str_equal(tag->name, parent_name))	/* prevent Foo::Foo from making parent = child */
-		{
-			GtkTreeIter *new_iter = g_new0(GtkTreeIter, 1);
+				/* walk parent candidates to find the better one.
+				 * if there are more than one, take the one that has the closest line number
+				 * after the tag we're searching the parent for */
+				candidates = g_hash_table_lookup(parents_table, parent_name);
+				if (candidates)
+				{
+					GList *node;
+					glong delta = G_MAXLONG;
+					foreach_list(node, *candidates)
+					{
+						TMTag *parent_tag;
+						glong  d;
 
-			/* set an iter value for the hash key */
-			g_hash_table_insert(parent_hash, tag->name, new_iter);
-			/* instead of ignoring the appended child iter below, use the one in the hash table */
-			child = new_iter;
-		}
+						gtk_tree_model_get(GTK_TREE_MODEL(store), node->data,
+								SYMBOLS_COLUMN_TAG, &parent_tag, -1);
 
-		if (!find_iter(tree_store, iter_hash, tag, child))
-		{
-			gboolean expand = FALSE;
+						d = tag->atts.entry.line - parent_tag->atts.entry.line;
+						if (! parent_search || (d >= 0 && d < delta))
+						{
+							delta = d;
+							parent_search = node->data;
+						}
+					}
+				}
+
+				if (parent_search)
+					parent = parent_search;
+				else
+					parent_name = NULL;
+			}
 
 			/* only expand to the iter if the parent was empty, otherwise we let the
 			 * folding as it was before (already expanded, or closed by the user) */
-			expand = !gtk_tree_model_iter_has_child(GTK_TREE_MODEL(tree_store), parent);
-			gtk_tree_store_append(tree_store, child, parent);
+			expand = ! gtk_tree_model_iter_has_child(model, parent);
+
+			/* insert the new element */
+			gtk_tree_store_append(store, &iter, parent);
+			name = get_symbol_name(doc, tag, parent_name != NULL);
+			tooltip = get_symbol_tooltip(doc, tag);
+			gtk_tree_store_set(store, &iter,
+					SYMBOLS_COLUMN_NAME, name,
+					SYMBOLS_COLUMN_TOOLTIP, tooltip,
+					SYMBOLS_COLUMN_ICON, icon,
+					SYMBOLS_COLUMN_TAG, tag,
+					-1);
+			g_free(tooltip);
+			if (G_LIKELY(icon))
+				g_object_unref(icon);
+
+			update_parents_table(parents_table, tag, parent_name, &iter);
+
 			if (expand)
-			{
-				GtkTreePath *path;
-
-				path = gtk_tree_model_get_path(GTK_TREE_MODEL(tree_store), child);
-				gtk_tree_view_expand_to_path(GTK_TREE_VIEW(doc->priv->tag_tree), path);
-				gtk_tree_path_free(path);
-			}
+				tree_view_expand_to_iter(GTK_TREE_VIEW(doc->priv->tag_tree), &iter);
 		}
-
-		name = get_symbol_name(doc, tag, (parent_name != NULL));
-		tooltip = get_symbol_tooltip(doc, tag);
-		gtk_tree_store_set(tree_store, child,
-			SYMBOLS_COLUMN_ICON, icon,
-			SYMBOLS_COLUMN_NAME, name,
-			SYMBOLS_COLUMN_TAG, tag,
-			SYMBOLS_COLUMN_VALID, TRUE,
-			SYMBOLS_COLUMN_TOOLTIP, tooltip,
-			-1);
-
-		g_free(tooltip);
-		if (G_LIKELY(G_IS_OBJECT(icon)))
-			g_object_unref(icon);
 	}
-	else
-		geany_debug("Missing symbol-tree parent iter for type %d!", tag->type);
-}
 
-
-static void add_tree_tags(GeanyDocument *doc, const GList *tags)
-{
-	const GList *item;
-	GHashTable *parent_hash;
-	GHashTable *iter_hash;
-
-	/* Create a hash table "parent_tag_name":(GtkTreeIter*) */
-	parent_hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
-	iter_hash = build_iter_table(doc->priv->tag_store);
-
-	/* find and store all parent names in the hash table */
-	for (item = tags; item; item = g_list_next(item))
-	{
-		const TMTag *tag = item->data;
-		const gchar *name = get_parent_name(tag, doc->file_type->id);
-
-		if (name)
-			g_hash_table_insert(parent_hash, (gpointer)name, NULL);
-	}
-	for (item = tags; item; item = g_list_next(item))
-	{
-		const TMTag *tag = item->data;
-
-		add_tree_tag(doc, tag, parent_hash, iter_hash);
-	}
-	g_hash_table_destroy(iter_hash);
-	g_hash_table_destroy(parent_hash);
+	g_hash_table_destroy(parents_table);
+	g_hash_table_destroy(tags_table);
 }
 
 
@@ -1392,16 +1553,17 @@ static gint compare_top_level_names(const gchar *a, const gchar *b)
 	guint i;
 	const gchar *name;
 
+	/* This should never happen as it would mean that two or more top
+	 * level items have the same name but it can happen by typos in the translations. */
+	if (utils_str_equal(a, b))
+		return 1;
+
 	foreach_ptr_array(name, i, top_level_iter_names)
 	{
 		if (utils_str_equal(name, a))
 			return -1;
 		if (utils_str_equal(name, b))
 			return 1;
-		/* This should never happen as it would mean that two or more top
-		 * level items have the same name but it can happen by typos in the translations. */
-		if (utils_str_equal(a, b))
-			return 1;
 	}
 	g_warning("Couldn't find top level node '%s' or '%s'!", a, b);
 	return 0;
@@ -1452,7 +1614,7 @@ static gint tree_sort_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 			/* this is what g_strcmp0() does */
 			if (! astr)
 				cmp = -(astr != bstr);
-			if (! bstr)
+			else if (! bstr)
 				cmp = astr != bstr;
 			else
 			{
@@ -1485,53 +1647,6 @@ static void sort_tree(GtkTreeStore *store, gboolean sort_by_name)
 }
 
 
-static void invalidate_rows(GtkTreeStore *store)
-{
-	GtkTreeIter iter;
-	GtkTreeModel *model = GTK_TREE_MODEL(store);
-
-	if (!gtk_tree_model_get_iter_first(model, &iter))
-		return;
-	do
-	{
-		gtk_tree_store_set(store, &iter, SYMBOLS_COLUMN_VALID, FALSE, -1);
-	}
-	while (next_iter(model, &iter));
-}
-
-
-static void remove_invalid_rows(GtkTreeStore *store)
-{
-	GtkTreeIter iter;
-	GtkTreeModel *model = GTK_TREE_MODEL(store);
-	gboolean cont;
-
-	cont = gtk_tree_model_get_iter_first(model, &iter);
-	while (cont)
-	{
-		gboolean valid;
-
-		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_VALID, &valid, -1);
-		if (!valid)
-		{
-			GtkTreeIter parent;
-			gboolean have_parent;
-
-			have_parent = gtk_tree_model_iter_parent(model, &parent, &iter);
-			cont = gtk_tree_store_remove(store, &iter);
-			/* if there is no next at this level but there is a parent iter, continue from it */
-			if (!cont && have_parent)
-			{
-				iter = parent;
-				cont = next_iter(model, &iter);
-			}
-		}
-		else
-			cont = next_iter(model, &iter);
-	}
-}
-
-
 gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 {
 	GList *tags;
@@ -1548,16 +1663,12 @@ gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 	 * models that are currently being built */
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(doc->priv->tag_store), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, 0);
 
-	/* invalidate all content */
-	invalidate_rows(doc->priv->tag_store);
-
 	/* add grandparent type iters */
 	add_top_level_items(doc);
 
-	add_tree_tags(doc, tags);
+	update_tree_tags(doc, &tags);
 	g_list_free(tags);
 
-	remove_invalid_rows(doc->priv->tag_store);
 	hide_empty_rows(doc->priv->tag_store);
 
 	if (sort_mode == SYMBOLS_SORT_USE_PREVIOUS)
@@ -1575,17 +1686,18 @@ gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 static GeanyFiletype *detect_global_tags_filetype(const gchar *utf8_filename)
 {
 	gchar *tags_ext;
-	gchar *shortname = g_strdup(utf8_filename);
+	gchar *shortname = utils_strdupa(utf8_filename);
 	GeanyFiletype *ft = NULL;
 
-	tags_ext = strstr(shortname, ".tags");
+	tags_ext = g_strrstr(shortname, ".tags");
 	if (tags_ext)
 	{
 		*tags_ext = '\0';	/* remove .tags extension */
 		ft = filetypes_detect_from_extension(shortname);
+		if (ft->id != GEANY_FILETYPES_NONE)
+			return ft;
 	}
-	g_free(shortname);
-	return ft;
+	return NULL;
 }
 
 
@@ -1596,9 +1708,8 @@ static GeanyFiletype *detect_global_tags_filetype(const gchar *utf8_filename)
  * CFLAGS=-I/home/user/libname-1.x geany -g libname.d.tags libname.h */
 int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess)
 {
-	/* -E pre-process, -dD output user macros, -p prof info (?),
-	 * -undef remove builtin macros (seems to be needed with FC5 gcc 4.1.1) */
-	const char pre_process[] = "gcc -E -dD -p -undef";
+	/* -E pre-process, -dD output user macros, -p prof info (?) */
+	const char pre_process[] = "gcc -E -dD -p -I.";
 
 	if (argc > 2)
 	{
@@ -1667,8 +1778,8 @@ void symbols_show_load_tags_dialog(void)
 		NULL);
 	gtk_widget_set_name(dialog, "GeanyDialog");
 	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, _("Geany tag files (*.tags)"));
-	gtk_file_filter_add_pattern(filter, "*.tags");
+	gtk_file_filter_set_name(filter, _("Geany tag files (*.*.tags)"));
+	gtk_file_filter_add_pattern(filter, "*.*.tags");
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
 
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
@@ -1701,15 +1812,12 @@ void symbols_show_load_tags_dialog(void)
 }
 
 
-/* Fills a hash table with filetype keys that hold a linked list of filenames. */
-static GHashTable *get_tagfile_hash(const GSList *file_list)
+static void detect_tag_files(const GSList *file_list)
 {
 	const GSList *node;
-	GHashTable *hash = g_hash_table_new(NULL, NULL);
 
 	for (node = file_list; node != NULL; node = g_slist_next(node))
 	{
-		GList *fnames;
 		gchar *fname = node->data;
 		gchar *utf8_fname = utils_get_utf8_from_locale(fname);
 		GeanyFiletype *ft = detect_global_tags_filetype(utf8_fname);
@@ -1717,52 +1825,42 @@ static GHashTable *get_tagfile_hash(const GSList *file_list)
 		g_free(utf8_fname);
 
 		if (FILETYPE_ID(ft) != GEANY_FILETYPES_NONE)
-		{
-			fnames = g_hash_table_lookup(hash, ft);	/* may be NULL */
-			fnames = g_list_append(fnames, fname);
-			g_hash_table_insert(hash, ft, fnames);
-		}
+			ft->priv->tag_files = g_slist_prepend(ft->priv->tag_files, fname);
 		else
 			geany_debug("Unknown filetype for file '%s'.", fname);
 	}
-	return hash;
 }
 
 
-static GHashTable *init_user_tags(void)
+static void init_user_tags(void)
 {
 	GSList *file_list = NULL, *list = NULL;
-	GHashTable *lang_hash;
 	gchar *dir;
 
-	dir = utils_build_path(app->configdir, "tags", NULL);
+	dir = g_build_filename(app->configdir, "tags", NULL);
 	/* create the user tags dir for next time if it doesn't exist */
 	if (! g_file_test(dir, G_FILE_TEST_IS_DIR))
 	{
 		utils_mkdir(dir, FALSE);
 	}
-	file_list = utils_get_file_list_full(dir, TRUE, TRUE, NULL);
+	file_list = utils_get_file_list_full(dir, TRUE, FALSE, NULL);
 
-	setptr(dir, utils_build_path(app->datadir, "tags", NULL));
-	list = utils_get_file_list_full(dir, TRUE, TRUE, NULL);
+	SETPTR(dir, g_build_filename(app->datadir, "tags", NULL));
+	list = utils_get_file_list_full(dir, TRUE, FALSE, NULL);
 	g_free(dir);
 
 	file_list = g_slist_concat(file_list, list);
-
-	lang_hash = get_tagfile_hash(file_list);
-	/* don't need to delete list contents because they are now used for hash contents */
+	detect_tag_files(file_list);
+	/* don't need to delete list contents because they are stored in ft->priv->tag_files */
 	g_slist_free(file_list);
-
-	return lang_hash;
 }
 
 
 static void load_user_tags(filetype_id ft_id)
 {
 	static guchar *tags_loaded = NULL;
-	static GHashTable *lang_hash = NULL;
-	GList *fnames;
-	const GList *node;
+	static gboolean init_tags = FALSE;
+	const GSList *node;
 	GeanyFiletype *ft = filetypes[ft_id];
 
 	g_return_if_fail(ft_id > 0);
@@ -1773,27 +1871,25 @@ static void load_user_tags(filetype_id ft_id)
 		return;
 	tags_loaded[ft_id] = TRUE;	/* prevent reloading */
 
-	if (lang_hash == NULL)
-		lang_hash = init_user_tags();
+	if (!init_tags)
+	{
+		init_user_tags();
+		init_tags = TRUE;
+	}
 
-	fnames = g_hash_table_lookup(lang_hash, ft);
-
-	for (node = fnames; node != NULL; node = g_list_next(node))
+	for (node = ft->priv->tag_files; node != NULL; node = g_slist_next(node))
 	{
 		const gchar *fname = node->data;
 
 		symbols_load_global_tags(fname, ft);
 	}
-	g_list_foreach(fnames, (GFunc) g_free, NULL);
-	g_list_free(fnames);
-	g_hash_table_remove(lang_hash, (gpointer) ft);
 }
 
 
 static gboolean goto_tag(const gchar *name, gboolean definition)
 {
 	const gint forward_types = tm_tag_prototype_t | tm_tag_externvar_t;
-	gint type;
+	guint type;
 	TMTag *tmtag = NULL;
 	GeanyDocument *old_doc = document_get_current();
 
@@ -2117,7 +2213,7 @@ static void on_symbol_tree_menu_show(GtkWidget *widget,
 
 static void on_expand_collapse(GtkWidget *widget, gpointer user_data)
 {
-	gboolean expand = utils_str_equal(user_data, GTK_STOCK_ADD);
+	gboolean expand = GPOINTER_TO_INT(user_data);
 	GeanyDocument *doc = document_get_current();
 
 	if (! doc)
@@ -2141,12 +2237,12 @@ static void create_taglist_popup_menu(void)
 	symbol_menu.expand_all = item = ui_image_menu_item_new(GTK_STOCK_ADD, _("_Expand All"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), (gpointer)GTK_STOCK_ADD);
+	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), GINT_TO_POINTER(TRUE));
 
 	symbol_menu.collapse_all = item = ui_image_menu_item_new(GTK_STOCK_REMOVE, _("_Collapse All"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), (gpointer)GTK_STOCK_REMOVE);
+	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), GINT_TO_POINTER(FALSE));
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -2174,7 +2270,7 @@ static void create_taglist_popup_menu(void)
 
 static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 {
-	gchar *f = utils_build_path(app->configdir, "ignore.tags", NULL);
+	gchar *f = g_build_filename(app->configdir, "ignore.tags", NULL);
 
 	g_return_if_fail(NZV(doc->real_path));
 
@@ -2191,7 +2287,7 @@ void symbols_init(void)
 
 	create_taglist_popup_menu();
 
-	f = utils_build_path(app->configdir, "ignore.tags", NULL);
+	f = g_build_filename(app->configdir, "ignore.tags", NULL);
 	ui_add_config_file_menu_item(f, NULL, NULL);
 	g_free(f);
 
@@ -2204,5 +2300,3 @@ void symbols_finalize(void)
 	g_strfreev(html_entities);
 	g_strfreev(c_tags_ignore);
 }
-
-
