@@ -1,8 +1,8 @@
 /*
  *      toolbar.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,13 +17,13 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: toolbar.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 #ifndef GEANY_TOOLBAR_H
 #define GEANY_TOOLBAR_H
 
+G_BEGIN_DECLS
+
 /** Toolbar settings. */
 typedef struct GeanyToolbarPrefs
 {
@@ -61,4 +61,6 @@ void toolbar_finalize(void);
 
 void toolbar_configure(GtkWindow *parent);
 
+G_END_DECLS
+
 #endif
