@@ -1,8 +1,8 @@
 /*
  *      utils.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: utils.c 5966 2011-09-28 19:49:45Z colombanw $
  */
 
 /*
@@ -94,7 +92,7 @@ void utils_open_browser(const gchar *uri)
 			if (new_cmd == NULL) /* user canceled */
 				again = FALSE;
 			else
-				setptr(tool_prefs.browser_cmd, new_cmd);
+				SETPTR(tool_prefs.browser_cmd, new_cmd);
 		}
 		g_free(cmdline);
 	}
@@ -444,6 +442,25 @@ gdouble utils_scale_round(gdouble val, gdouble factor)
 }
 
 
+/* like g_utf8_strdown() but if @str is not valid UTF8, convert it from locale first.
+ * returns NULL on charset conversion failure */
+static gchar *utf8_strdown(const gchar *str)
+{
+	gchar *down;
+
+	if (g_utf8_validate(str, -1, NULL))
+		down = g_utf8_strdown(str, -1);
+	else
+	{
+		down = g_locale_to_utf8(str, -1, NULL, NULL, NULL);
+		if (down)
+			SETPTR(down, g_utf8_strdown(down, -1));
+	}
+
+	return down;
+}
+
+
 /**
  *  A replacement function for g_strncasecmp() to compare strings case-insensitive.
  *  It converts both strings into lowercase using g_utf8_strdown() and then compare
@@ -469,30 +486,17 @@ gint utils_str_casecmp(const gchar *s1, const gchar *s2)
 	g_return_val_if_fail(s1 != NULL, 1);
 	g_return_val_if_fail(s2 != NULL, -1);
 
-	tmp1 = g_strdup(s1);
-	tmp2 = g_strdup(s2);
-
-	/* first ensure strings are UTF-8 */
-	if (! g_utf8_validate(s1, -1, NULL))
-		setptr(tmp1, g_locale_to_utf8(s1, -1, NULL, NULL, NULL));
-	if (! g_utf8_validate(s2, -1, NULL))
-		setptr(tmp2, g_locale_to_utf8(s2, -1, NULL, NULL, NULL));
-
-	if (tmp1 == NULL)
-	{
-		g_free(tmp2);
+	/* ensure strings are UTF-8 and lowercase */
+	tmp1 = utf8_strdown(s1);
+	if (! tmp1)
 		return 1;
-	}
-	if (tmp2 == NULL)
+	tmp2 = utf8_strdown(s2);
+	if (! tmp2)
 	{
 		g_free(tmp1);
 		return -1;
 	}
 
-	/* then convert the strings into a case-insensitive form */
-	setptr(tmp1, g_utf8_strdown(tmp1, -1));
-	setptr(tmp2, g_utf8_strdown(tmp2, -1));
-
 	/* compare */
 	result = strcmp(tmp1, tmp2);
 
@@ -590,7 +594,7 @@ gchar *utils_remove_ext_from_filename(const gchar *filename)
 {
 	gchar *last_dot;
 	gchar *result;
-	gint i;
+	gsize len;
 
 	g_return_val_if_fail(filename != NULL, NULL);
 
@@ -598,15 +602,11 @@ gchar *utils_remove_ext_from_filename(const gchar *filename)
 	if (! last_dot)
 		return g_strdup(filename);
 
-	/* assumes extension is small, so extra bytes don't matter */
-	result = g_malloc(strlen(filename));
-	i = 0;
-	while ((filename + i) != last_dot)
-	{
-		result[i] = filename[i];
-		i++;
-	}
-	result[i] = 0;
+	len = (gsize) (last_dot - filename);
+	result = g_malloc(len + 1);
+	memcpy(result, filename, len);
+	result[len] = 0;
+
 	return result;
 }
 
@@ -695,39 +695,16 @@ void utils_str_replace_all(gchar **haystack, const gchar *needle, const gchar *r
 
 gint utils_strpos(const gchar *haystack, const gchar *needle)
 {
-	gint haystack_length = strlen(haystack);
-	gint needle_length = strlen(needle);
-	gint i, j, pos = -1;
+	const gchar *sub;
 
-	if (needle_length > haystack_length)
-	{
+	if (! *needle)
 		return -1;
-	}
-	else
-	{
-		for (i = 0; (i < haystack_length) && pos == -1; i++)
-		{
-			if (haystack[i] == needle[0] && needle_length == 1)
-				return i;
-			else if (haystack[i] == needle[0])
-			{
-				for (j = 1; (j < needle_length); j++)
-				{
-					if (haystack[i + j] == needle[j])
-					{
-						if (pos == -1)
-							pos = i;
-					}
-					else
-					{
-						pos = -1;
-						break;
-					}
-				}
-			}
-		}
-		return pos;
-	}
+
+	sub = strstr(haystack, needle);
+	if (! sub)
+		return -1;
+
+	return sub - haystack;
 }
 
 
@@ -818,8 +795,7 @@ gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gch
 	gint tmp;
 	GError *error = NULL;
 
-	if (G_UNLIKELY(config == NULL))
-		return default_value;
+	g_return_val_if_fail(config, default_value);
 
 	tmp = g_key_file_get_integer(config, section, key, &error);
 	if (error)
@@ -849,8 +825,7 @@ gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const
 	gboolean tmp;
 	GError *error = NULL;
 
-	if (G_UNLIKELY(config == NULL))
-		return default_value;
+	g_return_val_if_fail(config, default_value);
 
 	tmp = g_key_file_get_boolean(config, section, key, &error);
 	if (error)
@@ -879,8 +854,7 @@ gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gc
 {
 	gchar *tmp;
 
-	if (G_UNLIKELY(config == NULL))
-		return g_strdup(default_value);
+	g_return_val_if_fail(config, g_strdup(default_value));
 
 	tmp = g_key_file_get_string(config, section, key, NULL);
 	if (!tmp)
@@ -1466,8 +1440,6 @@ GSList *utils_get_file_list_full(const gchar *path, gboolean full_path, gboolean
 	/* sorting last is quicker than on insertion */
 	if (sort)
 		list = g_slist_sort(list, (GCompareFunc) utils_str_casecmp);
-	else
-		list = g_slist_reverse(list);
 	return list;
 }
 
@@ -1551,7 +1523,7 @@ gint utils_string_find(GString *haystack, gint start, gint end, const gchar *nee
 
 
 /* Replaces @len characters from offset @a pos.
- * len can be -1 for str->len.
+ * len can be -1 to replace the remainder of @a str.
  * returns: pos + strlen(replace). */
 gint utils_string_replace(GString *str, gint pos, gint len, const gchar *replace)
 {
@@ -1561,7 +1533,6 @@ gint utils_string_replace(GString *str, gint pos, gint len, const gchar *replace
 		g_string_insert(str, pos, replace);
 		pos += strlen(replace);
 	}
-
 	return pos;
 }
 
@@ -1622,6 +1593,45 @@ guint utils_string_replace_first(GString *haystack, const gchar *needle, const g
 }
 
 
+/* Similar to g_regex_replace but allows matching a subgroup.
+ * match_num: which match to replace, 0 for whole match.
+ * literal: FALSE to interpret escape sequences in @a replace.
+ * returns: number of replacements.
+ * bug: replaced text can affect matching of ^ or \b */
+guint utils_string_regex_replace_all(GString *haystack, GRegex *regex,
+		guint match_num, const gchar *replace, gboolean literal)
+{
+	GMatchInfo *minfo;
+	guint ret = 0;
+	gint start = 0;
+
+	g_assert(literal); /* escapes not implemented yet */
+	g_return_val_if_fail(replace, 0);
+
+	/* ensure haystack->str is not null */
+	if (haystack->len == 0)
+		return 0;
+
+	/* passing a start position makes G_REGEX_MATCH_NOTBOL automatic */
+	while (g_regex_match_full(regex, haystack->str, -1, start, 0, &minfo, NULL))
+	{
+		gint end, len;
+
+		g_match_info_fetch_pos(minfo, match_num, &start, &end);
+		len = end - start;
+		utils_string_replace(haystack, start, len, replace);
+		ret++;
+
+		/* skip past whole match */
+		g_match_info_fetch_pos(minfo, 0, NULL, &end);
+		start = end - len + strlen(replace);
+		g_match_info_free(minfo);
+	}
+	g_match_info_free(minfo);
+	return ret;
+}
+
+
 /* Get project or default startup directory (if set), or NULL. */
 const gchar *utils_get_default_dir_utf8(void)
 {
@@ -1742,69 +1752,6 @@ gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFl
 }
 
 
-static gboolean utils_string_vappend(GString *buffer, const gchar *sep, va_list args)
-{
-	const gchar *str = va_arg(args, const gchar *);
-
-	if (!str)
-		return FALSE;
-	do
-	{
-		g_string_append(buffer, str);
-		str = va_arg(args, const gchar *);
-		if (str && sep)
-			g_string_append(buffer, sep);
-	}
-	while (str);
-
-	return TRUE;
-}
-
-
-/* Like g_build_path() but without first argument. */
-gchar *utils_build_path(const gchar *first, ...)
-{
-	static GString *buffer = NULL;
-	va_list args;
-
-	if (! buffer)
-		buffer = g_string_new(first);
-	else
-		g_string_assign(buffer, first);
-
-	g_string_append_c(buffer, G_DIR_SEPARATOR);
-
-	va_start(args, first);
-	utils_string_vappend(buffer, G_DIR_SEPARATOR_S, args);
-	va_end(args);
-	return g_strdup(buffer->str);
-}
-
-
-/* Concatenates a path with other strings.
- * @param path A path, which will have a separator added before the other strings.
- * @param ... Strings to concatenate (no directory separators will be
- * 	inserted between them).
- * E.g. filename = utils_make_filename(app->datadir, "filetypes.", ext, NULL); */
-gchar *utils_make_filename(const gchar *path, ...)
-{
-	static GString *buffer = NULL;
-	va_list args;
-
-	if (! buffer)
-		buffer = g_string_new(path);
-	else
-		g_string_assign(buffer, path);
-
-	g_string_append_c(buffer, G_DIR_SEPARATOR);
-
-	va_start(args, path);
-	utils_string_vappend(buffer, NULL, args);
-	va_end(args);
-	return g_strdup(buffer->str);
-}
-
-
 /* Retrieves the path for the given URI.
  * It returns:
  * - the path which was determined by g_filename_from_uri() or GIO
@@ -1856,7 +1803,6 @@ gboolean utils_is_remote_path(const gchar *path)
 		return TRUE;
 
 #ifndef G_OS_WIN32
-	if (glib_check_version(2, 16, 0) == NULL) /* no need to check for this with GLib < 2.16 */
 	{
 		static gchar *fuse_path = NULL;
 		static gsize len = 0;
@@ -1873,6 +1819,7 @@ gboolean utils_is_remote_path(const gchar *path)
 		return (strncmp(path, fuse_path, len) == 0);
 	}
 #endif
+
 	return FALSE;
 }
 
@@ -1893,6 +1840,10 @@ void utils_tidy_path(gchar *filename)
 	if (str->len >= 2 && strncmp(str->str, "\\\\", 2) == 0)
 		preserve_double_backslash = TRUE;
 
+#ifdef G_OS_WIN32
+	/* using MSYS we can get Unix-style separators */
+	utils_string_replace_all(str, "/", G_DIR_SEPARATOR_S);
+#endif
 	/* replace "/./" and "//" */
 	utils_string_replace_all(str, G_DIR_SEPARATOR_S "." G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S);
 	utils_string_replace_all(str, G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S);
@@ -1962,17 +1913,6 @@ gchar *utils_str_remove_chars(gchar *string, const gchar *chars)
 }
 
 
-static void utils_slist_remove_next(GSList *node)
-{
-	GSList *old = node->next;
-
-	g_return_if_fail(old);
-
-	node->next = old->next;
-	g_slist_free_1(old);
-}
-
-
 /* Gets list of sorted filenames with no path and no duplicates from user and system config */
 GSList *utils_get_config_files(const gchar *subdir)
 {
@@ -1984,7 +1924,7 @@ GSList *utils_get_config_files(const gchar *subdir)
 	{
 		utils_mkdir(path, FALSE);
 	}
-	setptr(path, g_build_path(G_DIR_SEPARATOR_S, app->datadir, subdir, NULL));
+	SETPTR(path, g_build_path(G_DIR_SEPARATOR_S, app->datadir, subdir, NULL));
 	syslist = utils_get_file_list_full(path, FALSE, FALSE, NULL);
 	/* merge lists */
 	list = g_slist_concat(list, syslist);
@@ -1995,8 +1935,11 @@ GSList *utils_get_config_files(const gchar *subdir)
 	{
 		if (node->next && utils_str_equal(node->next->data, node->data))
 		{
-			g_free(node->next->data);
-			utils_slist_remove_next(node);
+			GSList *old = node->next;
+
+			g_free(old->data);
+			node->next = old->next;
+			g_slist_free1(old);
 		}
 	}
 	g_free(path);
@@ -2028,7 +1971,7 @@ gchar *utils_get_help_url(const gchar *suffix)
 
 	if (suffix != NULL)
 	{
-		setptr(uri, g_strconcat(uri, suffix, NULL));
+		SETPTR(uri, g_strconcat(uri, suffix, NULL));
 	}
 
 	return uri;
