@@ -1,8 +1,8 @@
 /*
  *      win32.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  $Id: win32.c 5946 2011-09-22 15:53:14Z ntrel $
  */
 
 /*
@@ -77,7 +75,8 @@ typedef struct _geany_win32_spawn geany_win32_spawn;
 
 static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **error);
 static HANDLE GetTempFileHandle(GError **error);
-static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline, const TCHAR *dir, GError **error);
+static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline,
+		const TCHAR *dir, GError **error);
 static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **error);
 
 
@@ -158,14 +157,12 @@ static wchar_t *get_filters(gboolean project_files)
 	if (project_files)
 	{
 		string = g_strconcat(_("Geany project files"), "\t", "*." GEANY_PROJECT_EXT, "\t",
-			filetypes[GEANY_FILETYPES_NONE]->title, "\t",
-			filetypes[GEANY_FILETYPES_NONE]->pattern[0], "\t", NULL);
+			_("All files"), "\t", "*", "\t", NULL);
 	}
 	else
 	{
 		string = g_strconcat(_("Executables"), "\t", "*.exe;*.bat;*.cmd", "\t",
-			filetypes[GEANY_FILETYPES_NONE]->title, "\t",
-			filetypes[GEANY_FILETYPES_NONE]->pattern[0], "\t", NULL);
+			_("All files"), "\t", "*", "\t", NULL);
 	}
 
 	/* replace all "\t"s by \0 */
@@ -245,7 +242,7 @@ gchar *win32_show_folder_dialog(GtkWidget *parent, const gchar *title, const gch
 		parent = main_widgets.window;
 
 	memset(&bi, 0, sizeof bi);
-	bi.hwndOwner = GDK_WINDOW_HWND(parent->window);
+	bi.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(parent));
 	bi.pidlRoot = NULL;
 	bi.lpszTitle = w_title;
 	bi.lpfn = BrowseCallbackProc;
@@ -298,7 +295,7 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(parent->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(parent));
 	of.lpstrFilter = get_filters(project_file_filter);
 
 	of.lpstrCustomFilter = NULL;
@@ -360,7 +357,7 @@ gboolean win32_show_document_open_dialog(GtkWindow *parent, const gchar *title,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(parent)));
 	of.lpstrFilter = get_file_filters();
 
 	of.lpstrCustomFilter = NULL;
@@ -443,7 +440,7 @@ gchar *win32_show_document_save_as_dialog(GtkWindow *parent, const gchar *title,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(parent)));
 
 	of.lpstrFilter = get_file_filter_all_files();
 	of.lpstrCustomFilter = NULL;
@@ -499,7 +496,7 @@ gchar *win32_show_file_dialog(GtkWindow *parent, const gchar *title, const gchar
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(parent)));
 
 	of.lpstrFile = w_file;
 	of.nMaxFile = 2048;
@@ -535,10 +532,10 @@ void win32_show_font_dialog(void)
 
 	memset(&lf, 0, sizeof lf);
 	/* TODO: init lf members */
-	
+
 	memset(&cf, 0, sizeof cf);
 	cf.lStructSize = sizeof cf;
-	cf.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
+	cf.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window));
 	cf.lpLogFont = &lf;
 	/* support CF_APPLY? */
 	cf.Flags = CF_NOSCRIPTSEL | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
@@ -565,7 +562,7 @@ void win32_show_color_dialog(const gchar *colour)
 	/* Initialize CHOOSECOLOR */
 	memset(&cc, 0, sizeof cc);
 	cc.lStructSize = sizeof(cc);
-	cc.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
+	cc.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window));
 	cc.lpCustColors = (LPDWORD) acr_cust_clr;
 	cc.rgbResult = (colour != NULL) ? utils_strtod(colour, NULL, colour[0] == '#') : 0;
 	cc.Flags = CC_FULLOPEN | CC_RGBINIT;
@@ -613,7 +610,7 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(ui_widgets.prefs_dialog->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(ui_widgets.prefs_dialog));
 
 	of.lpstrFilter = get_filters(FALSE);
 	of.lpstrCustomFilter = NULL;
@@ -706,9 +703,9 @@ gboolean win32_message_dialog(GtkWidget *parent, GtkMessageType type, const gcha
 	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	if (parent != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(parent->window);
+		parent_hwnd = GDK_WINDOW_HWND(gtk_widget_get_window(parent));
 	else if (main_widgets.window != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
+		parent_hwnd = GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window));
 
 	/* display the message box */
 	rc = MessageBoxW(parent_hwnd, w_msg, w_title, t);
@@ -732,33 +729,6 @@ gint win32_check_write_permission(const gchar *dir)
 }
 
 
-/* Special dialog to ask for an action when closing an unsaved file */
-gint win32_message_dialog_unsaved(const gchar *msg)
-{
-	static wchar_t w_msg[512];
-	static wchar_t w_title[512];
-	HWND parent_hwnd = NULL;
-	gint ret;
-
-	/* convert the Unicode chars to wide chars */
-	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, G_N_ELEMENTS(w_msg));
-	MultiByteToWideChar(CP_UTF8, 0, _("Question"), -1, w_title, G_N_ELEMENTS(w_title));
-
-	if (main_widgets.window != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
-
-	ret = MessageBoxW(parent_hwnd, w_msg, w_title, MB_YESNOCANCEL | MB_ICONQUESTION);
-	switch (ret)
-	{
-		case IDYES: ret = GTK_RESPONSE_YES; break;
-		case IDNO: ret = GTK_RESPONSE_NO; break;
-		case IDCANCEL: ret = GTK_RESPONSE_CANCEL; break;
-	}
-
-	return ret;
-}
-
-
 /* Just a simple wrapper function to open a browser window */
 void win32_open_browser(const gchar *uri)
 {
@@ -815,6 +785,13 @@ static void debug_setup_console()
 	fp = _fdopen(hConHandle, "w");
 	*stderr = *fp;
 	setvbuf(stderr, NULL, _IONBF, 0);
+
+	/* redirect unbuffered STDIN to the console */
+	lStdHandle = (long)GetStdHandle(STD_INPUT_HANDLE);
+	hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
+	fp = _fdopen( hConHandle, "r" );
+	*stdin = *fp;
+	setvbuf(stdin, NULL, _IONBF, 0);
 }
 
 
@@ -822,8 +799,8 @@ void win32_init_debug_code(void)
 {
 	if (app->debug_mode)
 	{
-		/* create a console window to get log messages on Windows */
-		/** TODO remove me? */
+		/* create a console window to get log messages on Windows,
+		 * especially useful when generating tags files */
 		debug_setup_console();
 		/* Enable GLib process spawn debug mode when Geany was started with the debug flag */
 		g_setenv("G_SPAWN_WIN32_DEBUG", "1", FALSE);
@@ -898,8 +875,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 		{
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("win32_spawn: Second CreateFile failed (%d)", (gint) GetLastError());
-			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+			g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 			g_free(msg);
 			return FALSE;
 		}
@@ -912,8 +888,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 		{
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("win32_spawn: Second CreateFile failed (%d)", (gint) GetLastError());
-			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+			g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 			g_free(msg);
 			return FALSE;
 		}
@@ -934,8 +909,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stdout pipe creation failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -948,8 +922,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stderr pipe creation failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -962,8 +935,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stdin pipe creation failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -981,8 +953,6 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	if (! fSuccess)
 	{
 		geany_debug("win32_spawn: Create process failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "Create process failed");
 		return FALSE;
 	}
 
@@ -1028,8 +998,7 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: Alloc failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_SPAWN_ERROR, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -1039,8 +1008,7 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: Cannot read tempfile");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -1049,8 +1017,7 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: CloseHandle failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		g_free(buffer);
 		*content = NULL;
@@ -1073,7 +1040,8 @@ gchar *win32_expand_environment_variables(const gchar *str)
 }
 
 
-static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline, const TCHAR *dir, GError **error)
+static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline,
+		const TCHAR *dir, GError **error)
 {
 	PROCESS_INFORMATION piProcInfo;
 	STARTUPINFOW siStartInfo;
@@ -1118,28 +1086,25 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("CreateChildProcess: CreateProcess failed (%s)", msg);
-		if (*error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
 	else
 	{
 		gint i;
-		DWORD dwStatus;
 
-		for (i = 0; i < 2 && (dwStatus = WaitForSingleObject(piProcInfo.hProcess, 30*1000)) == WAIT_TIMEOUT; i++)
+		for (i = 0; i < 2 && WaitForSingleObject(piProcInfo.hProcess, 30*1000) == WAIT_TIMEOUT; i++)
 		{
 			geany_debug("CreateChildProcess: CreateProcess failed");
 			TerminateProcess(piProcInfo.hProcess, WAIT_TIMEOUT); /* NOTE: This will not kill grandkids. */
 		}
 
-		if (GetExitCodeProcess(piProcInfo.hProcess, &gw_spawn->dwExitCode) != 0)
+		if (!GetExitCodeProcess(piProcInfo.hProcess, &gw_spawn->dwExitCode))
 		{
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("GetExitCodeProcess failed: %s", msg);
-			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
+			g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 			g_free(msg);
 		}
 		CloseHandle(piProcInfo.hProcess);
@@ -1161,8 +1126,7 @@ static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("ReadFromPipe: Closing handle failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return;
 	}
@@ -1197,8 +1161,7 @@ static HANDLE GetTempFileHandle(GError **error)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileHandle: GetTempPath failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1212,8 +1175,7 @@ static HANDLE GetTempFileHandle(GError **error)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileName failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1230,8 +1192,7 @@ static HANDLE GetTempFileHandle(GError **error)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileHandle: Second CreateFile failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1313,7 +1274,7 @@ gchar *win32_get_shortcut_target(const gchar *file_name)
 	gchar *path = NULL;
 	wchar_t *wfilename = g_utf8_to_utf16(file_name, -1, NULL, NULL, NULL);
 
-	resolve_link(GDK_WINDOW_HWND(main_widgets.window->window), wfilename, &path);
+	resolve_link(GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window)), wfilename, &path);
 	g_free(wfilename);
 
 	if (path == NULL)
