@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <ctype.h>
+#include <mio/mio.h>
 
 #include "parse.h"
 #include "vstring.h"
@@ -72,9 +73,9 @@ typedef struct sInputFile {
     vString	*path;		/* path of input file (if any) */
     vString	*line;		/* last line read from file */
     const unsigned char* currentLine;	/* current line being worked on */
-    FILE	*fp;		/* stream used for reading the file */
+    MIO		*mio;		/* stream used for reading the file */
     unsigned long lineNumber;	/* line number in the input file */
-    fpos_t	filePosition;	/* file position of current line */
+    MIOPos	filePosition;	/* file position of current line */
     int		ungetch;	/* a single character that was ungotten */
     boolean	eof;		/* have we reached the end of file? */
     boolean	newLine;	/* will the next character begin a new line? */
@@ -101,11 +102,6 @@ extern CONST_FILE inputFile File;
 /*
 *   FUNCTION PROTOTYPES
 */
-#ifdef NEED_PROTO_FGETPOS
-extern int fgetpos  (FILE *stream, fpos_t *pos);
-extern int fsetpos  (FILE *stream, const fpos_t *pos);
-#endif
-
 extern void freeSourceFileResources (void);
 extern boolean fileOpen (const char *const fileName, const langType language);
 extern boolean fileEOF (void);
@@ -113,8 +109,11 @@ extern void fileClose (void);
 extern int fileGetc (void);
 extern void fileUngetc (int c);
 extern const unsigned char *fileReadLine (void);
-extern char *readLine (vString *const vLine, FILE *const fp);
-extern char *readSourceLine (vString *const vLine, fpos_t location, long *const pSeekValue);
+extern char *readLine (vString *const vLine, MIO *const mio);
+extern char *readSourceLine (vString *const vLine, MIOPos location, long *const pSeekValue);
+extern boolean bufferOpen (unsigned char *buffer, int buffer_size,
+			   const char *const fileName, const langType language );
+#define bufferClose fileClose
 
 #endif	/* _READ_H */
 
